/*
 * Chax.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax.h"
#import "iChat.h"
#import "NSPreferences.h"

#import "LogViewerController.h"
#import "ActivityWindowController.h"
#import "UpdateController.h"
#import "Hax_Preferences.h"

#import "HaxAIMLGenerator.h"
#import "HaxAnimatingTableView.h"
#import "HaxBuddyPicture.h"
#import "HaxChatController.h"
#import "HaxChatDoc.h"
#import "HaxChatInputLine.h"
#import "HaxChatTextView.h"
#import "HaxFezPreferences.h"
#import "HaxPeopleList.h"
#import "HaxPeopleListController.h"
#import "HaxPeopleListControllerWindow.h"
#import "HaxPeopleListItem.h"
#import "HaxSecureWindow.h"
#import "HaxStagedFileNotifier.h"
#import "HaxVCLayoutController.h"
#import "TabbedChatController.h"

BOOL ShouldAutoAcceptFilesFromSender(Presentity *sender)
{
	BOOL accept = NO;
	if ([Chax boolForKey:@"AutoAcceptFiles"]) {
		NSString *group = [Chax objectForKey:@"AutoAcceptFiles.Group"];
		if (group != nil && ![group isEqualToString:@"none"]) {
			//Check the group
			NSArray *groups = [[sender abPerson] parentGroups];
			NSEnumerator *enumerator = [groups objectEnumerator];
			ABGroup *nextGroup = nil;
			
			while ( (nextGroup = [enumerator nextObject]) ) {
				if ([group isEqualToString:[nextGroup uniqueId]]) {
					accept = YES;
				}
			}
		}
		
		//Check the services
		if (!accept) {
			NSString *serviceName = [[sender service] shortName];
			
			if ([serviceName isEqualToString:@"AIM"] && [Chax boolForKey:@"AutoAcceptFiles.AIM"]) {
				accept = YES;
			} else if ([serviceName isEqualToString:@"Jabber"] && [Chax boolForKey:@"AutoAcceptFiles.Jabber"]) {
				accept = YES;
			} else if ([serviceName isEqualToString:@"Bonjour"] && [Chax boolForKey:@"AutoAcceptFiles.Bonjour"]) {
				accept = YES;
			}
		}
	}
	return accept;
}

@implementation Chax

+ (void)load
{
	//if ([[[NSBundle mainBundle] bundleIdentifier] isEqualToString:@"com.apple.iChat"] && [[NSDate date] compare:[NSDate dateWithNaturalLanguageString:@"2006-04-20"]] != NSOrderedDescending) {
	if ([[[NSBundle mainBundle] bundleIdentifier] isEqualToString:@"com.apple.iChat"]) {
		[HaxAIMLGenerator poseAsClass:[AIMLGenerator class]];
		[HaxAnimatingTableView poseAsClass:[AnimatingTableView class]];
		[HaxBuddyPicture poseAsClass:[BuddyPicture class]];
		[HaxChatController poseAsClass:[ChatController class]];
		[HaxChatDoc poseAsClass:[ChatDoc class]];
		[HaxChatInputLine poseAsClass:[ChatInputLine class]];
		[HaxChatTextView poseAsClass:[ChatTextView class]];
		[HaxFezPreferences poseAsClass:[NSPreferences class]];
		[HaxPeopleList poseAsClass:[PeopleList class]];
		[HaxPeopleListController poseAsClass:[PeopleListController class]];
		[HaxPeopleListControllerWindow poseAsClass:[PeopleListControllerWindow class]];
		[HaxPeopleListItem poseAsClass:[PeopleListItem class]];
		[HaxSecureWindow poseAsClass:[SecureWindow class]];
		[HaxStagedFileNotifier poseAsClass:[StagedFileNotifier class]];
		[HaxVCLayoutController poseAsClass:[VCLayoutController class]];
		
		//Migrate preferences if necessary
		if ([Chax objectForKey:@"DockForegroundColor"] == nil && [[NSUserDefaults standardUserDefaults] objectForKey:@"AutoUpdateChax"]) {
			[Hax_Preferences migratePreferences];
		}
		
		//Set the KnockKnock hidden pref to keep in sync with SkipNewMessageNotification
		[Prefs setKnockKnock:!([Chax boolForKey:@"SkipNewMessageNotification"] || [Chax boolForKey:@"EnableTabs"])];
		
		//Default auto update setting to true
		if ([Chax objectForKey:@"AutoUpdateChax"] == nil) {
			[Chax setBool:YES forKey:@"AutoUpdateChax"];
			[[UpdateController sharedController] setAutomaticCheckingEnabled:YES];
		}
		
		//Default tab icon showing to true
		if ([Chax objectForKey:@"TabShowUserIcons"] == nil) {
			[Chax setBool:YES forKey:@"TabShowUserIcons"];
		}
		
		//Default menu badge showing to true
		if ([Chax objectForKey:@"ShowMenuBadge"] == nil) {
			[Chax setBool:YES forKey:@"ShowMenuBadge"];
		}
		
		//Install time for donation sheet
		if ([Chax objectForKey:@"Install"] == nil) {
			[Chax setInteger:[[NSDate date] timeIntervalSinceReferenceDate] forKey:@"Install"];
		}
		
		NSMenu *viewMenu = [[[NSApp mainMenu] itemAtIndex:3] submenu];
		NSMenu *buddiesMenu = [[[NSApp mainMenu] itemAtIndex:4] submenu];
		const unichar leftArrowChar = NSLeftArrowFunctionKey;
		const unichar rightArrowChar = NSRightArrowFunctionKey;
		const unichar upArrowChar = NSUpArrowFunctionKey;
		const unichar downArrowChar = NSDownArrowFunctionKey;
		BOOL badge = [Chax boolForKey:@"ShowMenuBadge"];
		
		NSImage *badgeImage = [[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"menu-badge"]];
		
		//Toggle text status menu item
		id <NSMenuItem> menuItem = [viewMenu insertItemWithTitle:LocalizedString(@"show_text_status") action:@selector(toggleTextStatus:) keyEquivalent:@"" atIndex:1];
		if (badge) {
			[menuItem setImage:badgeImage];
		}
		[menuItem setTag:13371];
		[menuItem setState:NSOnState];
		
		//SMS message menu item
		menuItem = [buddiesMenu insertItemWithTitle:LocalizedString(@"send_sms") action:@selector(chaxMenuAction:) keyEquivalent:@"" atIndex:13];
		if (badge) {
			[menuItem setImage:badgeImage];
		}
		[menuItem setTag:91122];
		
		//More info menu item
		menuItem = [buddiesMenu insertItemWithTitle:LocalizedString(@"more_info") action:@selector(chaxMenuAction:) keyEquivalent:@"I" atIndex:1];
		[menuItem setTag:91121];
		[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask | NSAlternateKeyMask];
		
		//Always on top menu item
		menuItem = [[NSApp windowsMenu] insertItemWithTitle:LocalizedString(@"always_on_top") action:@selector(toggleIsOnTop:) keyEquivalent:@"" atIndex:2];
		if (badge) {
			[menuItem setImage:badgeImage];
		}
		[menuItem setTag:13374];
		
		//Log viewer menu item
		menuItem = [[NSApp windowsMenu] addItemWithTitle:LocalizedString(@"log_viewer") action:@selector(showWindow:) keyEquivalent:@""];
		if (badge) {
			[menuItem setImage:badgeImage];
		}
		[menuItem setTag:13373];
		[menuItem setTarget:[LogViewerController sharedController]];
		
		//Activity window menu item
		menuItem = [[NSApp windowsMenu] addItemWithTitle:LocalizedString(@"activity") action:@selector(showWindow:) keyEquivalent:@"a"];
		if (badge) {
			[menuItem setImage:badgeImage];
		}
		[menuItem setTag:13372];
		[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSAlternateKeyMask];
		[menuItem setTarget:[ActivityWindowController sharedController]];
		
		//Unified contact list menu item
		/*menuItem = [[NSApp windowsMenu] addItemWithTitle:LocalizedString(@"unified") action:@selector(showWindow:) keyEquivalent:@"4"];
		if (badge) {
			[menuItem setImage:badgeImage];
		}
		[menuItem setTag:13390];
		[menuItem setTarget:[UnifiedList sharedInstance]];*/
		
		//Tab navigation menu items
		[[NSApp windowsMenu] insertItem:[NSMenuItem separatorItem] atIndex:3];
		menuItem = [[NSApp windowsMenu] insertItemWithTitle:LocalizedString(@"select_prev_tab") action:@selector(previousTab:) keyEquivalent:[NSString stringWithCharacters:&leftArrowChar length:1] atIndex:4];
		if (badge) {
			[menuItem setImage:badgeImage];
		}
		[menuItem setTag:13381];
		
		NSMenuItem *menuItem2 = [[NSApp windowsMenu] insertItemWithTitle:LocalizedString(@"select_next_tab") action:@selector(nextTab:) keyEquivalent:[NSString stringWithCharacters:&rightArrowChar length:1] atIndex:5];
		if (badge) {
			[menuItem2 setImage:badgeImage];
		}
		[menuItem2 setTag:13380];
		
		switch ([Chax integerForKey:@"TabKeyEquivalent"]) {
			case 0:
				[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
				[menuItem2 setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
				break;
			case 2:
				[menuItem setKeyEquivalent:@"{"];
				[menuItem2 setKeyEquivalent:@"}"];
				break;
			case 3:
				[menuItem setKeyEquivalent:@"["];
				[menuItem2 setKeyEquivalent:@"]"];
				break;
			case 5:
				[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
				[menuItem2 setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
			case 4:
				[menuItem setKeyEquivalent:[NSString stringWithCharacters:&upArrowChar length:1]];
				[menuItem2 setKeyEquivalent:[NSString stringWithCharacters:&downArrowChar length:1]];
				break;
		}
		
		//Load up the first-load screen if needed
		/*if (![Chax boolForKey:@"HackWasSetup"]) {
			[Chax setBool:YES forKey:@"HackWasSetup"];
		}*/
		
		[badgeImage release];
	} else {
		NSLog(@"This beta of Chax has expired. Please check http://www.ksuther.com/chax/ for a newer version.");
	}
}

+ (BOOL)boolForKey:(NSString *)key
{
	return CFPreferencesGetAppBooleanValue((CFStringRef)key, CFSTR("com.ksuther.chax"), nil);
}

+ (int)integerForKey:(NSString *)key
{
	return CFPreferencesGetAppIntegerValue((CFStringRef)key, CFSTR("com.ksuther.chax"), nil);
}

+ (NSData *)dataForKey:(NSString *)key
{
	id object = (id)CFPreferencesCopyAppValue((CFStringRef)key, CFSTR("com.ksuther.chax"));
	if (object && CFGetTypeID(object) == CFDataGetTypeID()) {
		return [object autorelease];
	} else {
		[object release];
		return nil;
	}
}

+ (NSString *)stringForKey:(NSString *)key
{
	id object = (id)CFPreferencesCopyAppValue((CFStringRef)key, CFSTR("com.ksuther.chax"));
	if (object && CFGetTypeID(object) == CFStringGetTypeID()) {
		return [object autorelease];
	} else {
		[object release];
		return nil;
	}
}

+ (id)objectForKey:(NSString *)key
{
	return [(id)CFPreferencesCopyAppValue((CFStringRef)key, CFSTR("com.ksuther.chax")) autorelease];
}

+ (void)setBool:(BOOL)value forKey:(NSString *)key
{
	CFPreferencesSetAppValue((CFStringRef)key, (CFPropertyListRef)(value ? kCFBooleanTrue : kCFBooleanFalse), CFSTR("com.ksuther.chax"));
}

+ (void)setInteger:(int)value forKey:(NSString *)key
{
	CFPreferencesSetAppValue((CFStringRef)key, (CFPropertyListRef)[NSNumber numberWithInt:value], CFSTR("com.ksuther.chax"));
}

+ (void)setObject:(id)value forKey:(NSString *)key
{
	CFPreferencesSetAppValue((CFStringRef)key, (CFPropertyListRef)value, CFSTR("com.ksuther.chax"));
}

@end