/*
 * FloatingInfoWindow.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "FloatingInfoWindow.h"
#import "FloatingInfoView.h"
#import "URLTextView.h"

@implementation FloatingInfoWindow

- (id)initWithContentRect:(NSRect)contentRect styleMask:(unsigned int)styleMask backing:(NSBackingStoreType)bufferingType defer:(BOOL)deferCreation
{
	if ( (self = [super initWithContentRect:contentRect styleMask:NSBorderlessWindowMask backing:bufferingType defer:deferCreation]) ) {
		[self setContentView:[[FloatingInfoView alloc] initWithFrame:contentRect]];
		[self setLevel:NSPopUpMenuWindowLevel];
		[self setBackgroundColor:[NSColor clearColor]];
		[self setHasShadow:YES];
        [self setOpaque:NO];
		
		//Create an image view for the icon
		_imageView = [[NSImageView alloc] initWithFrame:NSMakeRect(15, 15, 90, 90)];
		[_imageView setImageFrameStyle:NSImageFrameNone];
		[_imageView setAnimates:YES];
		[_imageView setImageScaling:NSScaleToFit];
		[_imageView setAutoresizingMask:NSViewMinYMargin];
		[[self contentView] addSubview:[_imageView autorelease]];
		
		//Create a text view for the status message
		NSScrollView *scrollView = [[NSScrollView alloc] initWithFrame:NSMakeRect(112, 15, 230, 90)];
		_textView = [[URLTextView alloc] initWithFrame:NSMakeRect(0, 0, 230, 90)];
		[_textView setDrawsBackground:NO];
		[_textView setEditable:NO];
		[_textView setTextColor:[NSColor whiteColor]];
		
		[[scrollView contentView] setDocumentView:[_textView autorelease]];
		[scrollView setDrawsBackground:NO];
		
		[scrollView setAutoresizingMask:NSViewHeightSizable];
		[[self contentView] addSubview:[scrollView autorelease]];
	}
	return self;
}

- (void)dealloc
{
	[super dealloc];
}

- (BOOL)canBecomeMainWindow
{
	return YES;
}

- (BOOL)canBecomeKeyWindow
{
	return YES;
}

- (void)setImage:(NSImage *)image
{
	[_imageView setImage:image];
}

- (void)setString:(NSString *)string
{
	[_textView setString:string];
	[_textView highlightLinks];
}

- (void)repositionWithRect:(NSRect)rect
{
	NSRect windowFrame = [self frame], textFrame = [_textView frame], screenFrame = NSZeroRect;
	NSRect size = [[_textView textStorage] boundingRectWithSize:NSMakeSize(220, 0) options:NSStringDrawingUsesLineFragmentOrigin];
	
	//this is really ugly, but I'm feeling lazy
	NSEnumerator *enumerator;
	NSScreen *nextScreen;
	NSRect testRect = rect;
	do {
		enumerator = [[NSScreen screens] objectEnumerator];
		while ( (nextScreen = [enumerator nextObject]) ) {
			if (NSContainsRect([nextScreen frame], testRect)) {
				screenFrame = [nextScreen frame];
			}
		}
		testRect = NSOffsetRect(testRect, -5, 0);
	} while (NSEqualRects(screenFrame, NSZeroRect));
	
	if (size.size.height > 90) {
		//Increase the window and text view height by delta pixels
		float newHeight = size.size.height + 30;
		rect.origin.y -= newHeight - 120;
		windowFrame.size.height = newHeight;
	} else {
		windowFrame.size.height = 120;
	}
	
	//Figure out if this point will cause the window to display off the right side of the screen
	if (rect.origin.x + 350 > screenFrame.origin.x + screenFrame.size.width) {
		//Display to the left of the contact list instead
		rect.origin.x -= 320 + rect.size.width;
	}
	
	//if the bottom of the window is offscreen, move the window up
	if (rect.origin.y < screenFrame.origin.y) {
		//Display to the left of the contact list instead
		rect.origin.y = 15;
	}
	
	[self setFrame:NSMakeRect(rect.origin.x, rect.origin.y, windowFrame.size.width, windowFrame.size.height) display:YES animate:YES];
}

@end
