/*
 * HaxAnimatingTableView.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "HaxAnimatingTableView.h"
#import "Hax_Presentity.h"
#import "Chax.h"
#import "LogViewerController.h"
#import "InfoWindowController.h"

@interface HaxAnimatingTableView (PrivateHax)
- (NSMenu *)presentityMenuWithTag:(int)tag;
@end

@implementation HaxAnimatingTableView

- (NSMenu *)menuForEvent:(NSEvent *)event
{
	NSMenu *menu = [super menuForEvent:event], *pMenu;
	if ([menu numberOfItems] > 3) {
		NSArray *presentities = [[[self delegate] presentityAtRow:[self selectedRow]] presentities];
		id <NSMenuItem> menuItem;
		
		if ([[menu itemAtIndex:0] tag] == 91020) {
			[[menu itemAtIndex:0] setSubmenu:nil];
			[[menu itemAtIndex:0] setTag:0];
			[[menu itemAtIndex:3] setSubmenu:nil];
			[[menu itemAtIndex:3] setTag:0];
		}
		
		int onlinePresentities = 0;
		int i;
		
		for (i = 0; i < [presentities count]; i++) {
			if ([[presentities objectAtIndex:i] status] != 0 && [[presentities objectAtIndex:i] status] != 1) {
				onlinePresentities++;
			}
		}
		
		if (onlinePresentities > 1) {
			NSEnumerator *enumerator = [presentities objectEnumerator];
			id nextObject;
			int number = 0;
			while ( (nextObject = [enumerator nextObject]) ) {
				if ([nextObject status] != 1) {
					number++;
				}
			}
			
			if (number > 1) {
				menuItem = [menu itemAtIndex:0];
				[menuItem setSubmenu:[self presentityMenuWithTag:1]];
				[menuItem setTag:91020];
				
				menuItem = [menu itemAtIndex:3];
				[menuItem setSubmenu:[self presentityMenuWithTag:2]];
				[menuItem setTag:91021];
			}
		}
		
		//Remove the previous "More Info" and "View Logs" menu items
		menuItem = [menu itemWithTag:91120];
		if (menuItem) {
			[menu removeItem:menuItem];
		}
		
		menuItem = [menu itemWithTag:91121];
		if (menuItem) {
			[menu removeItem:menuItem];
		}
		
		menuItem = [menu itemWithTag:91122];
		if (menuItem) {
			[menu removeItem:menuItem];
		}
		
		menuItem = [menu addItemWithTitle:LocalizedString(@"more_info") action:@selector(chaxMenuAction:) keyEquivalent:@""];
		[menuItem setTag:91121];
		[menuItem setTarget:self];
		
		menuItem = [menu addItemWithTitle:LocalizedString(@"view_logs") action:@selector(chaxMenuAction:) keyEquivalent:@""];
		[menuItem setTag:91120];
		[menuItem setTarget:self];
		
		menuItem = [menu insertItemWithTitle:LocalizedString(@"send_sms") action:@selector(chaxMenuAction:) keyEquivalent:@"" atIndex:[menu numberOfItems] - 6];
		[menuItem setTag:91122];
		[menuItem setTarget:self];
	}
	return menu;
}

- (NSMenu *)presentityMenuWithTag:(int)tag
{
	NSMenu *menu = [[NSMenu alloc] init];
	Presentity *mainPresentity = [[self delegate] presentityAtRow:[self selectedRow]];
	NSEnumerator *enumerator = [[mainPresentity presentities] objectEnumerator];
	id <NSMenuItem> menuItem;
	id nextObject;
	
	while ( (nextObject = [enumerator nextObject]) ) {
		if ([nextObject status] != 0 && [nextObject status] != 1) {
			menuItem = [menu addItemWithTitle:[nextObject ID] action:@selector(chaxMenuAction:) keyEquivalent:@""];
			[menuItem setTag:tag];
			[menuItem setTarget:self];
			[menuItem setRepresentedObject:[NSArray arrayWithObjects:mainPresentity, nextObject, nil]];
		}
	}
	return [menu autorelease];
}

- (void)chaxMenuAction:(id)sender
{
	[[[sender representedObject] objectAtIndex:0] setBestPresentity:[[sender representedObject] objectAtIndex:1]];
	switch ([sender tag]) {
		case 1:
			[[self delegate] startChat:[[sender representedObject] objectAtIndex:1]];
			break;
		case 2:
			[(PeopleList *)[self delegate] sendMessage:[[sender representedObject] objectAtIndex:1]];
			break;
		case 91120:
			[[LogViewerController sharedController] window];
			[[LogViewerController sharedController] showLogsForPresentity:[[self delegate] presentityAtRow:[self selectedRow]]];
			break;
		case 91121:
			{
				NSRect frame = [self frame];
				NSPoint screenPoint = [[self window] convertBaseToScreen:[self convertPoint:frame.origin toView:nil]];
				screenPoint.y -= ([self rowHeight] * [self selectedRow]) + 90;
				screenPoint.x += [self frame].size.width - 15;
				frame.origin.x = screenPoint.x;
				frame.origin.y = screenPoint.y;
				frame.size.height = [self rowHeight];
				
				InfoWindowController *info = [InfoWindowController sharedController];
				[info setPresentity:[[self delegate] presentityAtRow:[self selectedRow]] rowRect:frame];
				[info showWindow:self];
			}
			break;
		case 91122:
			{
				Presentity *presentity = [[self delegate] presentityAtRow:[self selectedRow]];
				ABPerson *person = [presentity abPerson];
				
				if (person) {
					ABMultiValue *phones = nil;
					NSString *phone = nil;
					int i;
					
					phones = [person valueForProperty:kABPhoneProperty];
					
					for (i = 0; i < [phones count]; i++) {
						if ([[phones labelAtIndex:i] isEqualToString:kABPhoneMobileLabel]) {
							phone = [phones valueAtIndex:i];
							
							NSScanner *scanner = [NSScanner scannerWithString:phone];
							NSMutableString *strippedNumber = [[NSMutableString alloc] initWithString:@""];
							NSString *number;
							
							while (![scanner isAtEnd]) {
								NSString *next = nil;
								[scanner scanCharactersFromSet:[NSCharacterSet decimalDigitCharacterSet] intoString:&next];
								if (next) {
									[strippedNumber appendString:next];
								}
								
								if ([scanner scanLocation] < [phone length]) {
									[scanner setScanLocation:[scanner scanLocation] + 1];
								}
							}
							
							//strip out a leading 1 (long distance) if there is one
							if ([strippedNumber characterAtIndex:0] == '1') {
								number = [strippedNumber substringFromIndex:1];
							} else {
								number = [NSString stringWithString:strippedNumber];
							}
							
							if ([number length] != 10) {
								//this number doesn't seem to be a valid US phone number, or there's no area code given
								NSRunAlertPanel(LocalizedString(@"invalidsmsnumber_title"), LocalizedString(@"invalidsmsnumber_msg"), nil, nil, nil, phone);
								[strippedNumber release];
								break;
							}
							
							number = [@"+1" stringByAppendingString:number];
							
							//I don't think this presentity object is being released in some cases. Autoreleasing it will cause a double free in some instances.
							Presentity *mobilePresentity = [[Presentity alloc] initWithService:[Service aimService] ID:number];
							[[Fezz sharedInstance] createChatUIWithPresentities:[NSArray arrayWithObject:mobilePresentity] style:2];
							[strippedNumber release];
							break;
						}
					}
				}
			}
			break;
	}
}

- (BOOL)validateMenuItem:(id)fp8
{
	BOOL value = NO;
	
	switch ([fp8 tag]) {
		case 91122:
			{
				Presentity *presentity = [[self delegate] presentityAtRow:[self selectedRow]];
				ABPerson *person = [presentity abPerson];
				
				if (person) {
					ABMultiValue *phones = nil;
					NSString *phone = nil;
					int i;
					
					phones = [person valueForProperty:kABPhoneProperty];
					
					for (i = 0; i < [phones count]; i++) {
						if ([[phones labelAtIndex:i] isEqualToString:kABPhoneMobileLabel]) {
							value = ([[Service aimService] serviceLoginStatus] != 0);
							break;
						}
					}
				}
			}
			break;
		default:
			value = [super validateMenuItem:fp8];
	}
	
	return value;
}

- (void)reloadData
{
	if (_timer != nil) {
		[self performSelector:@selector(reloadData) withObject:nil afterDelay:1.0];
	} else {
		[super reloadData];
	}
}

- (void)_finishAnimation
{
	[super _finishAnimation];
	[[NSNotificationCenter defaultCenter] performSelector:@selector(postNotification:) withObject:[NSNotification notificationWithName:@"ReloadContactList" object:nil] afterDelay:0.2];
}

- (void)_drawGroupRowInRect:(struct _NSRect)fp8 withTitle:(id)fp24 andNumber:(id)fp28 mouseOver:(BOOL)fp32 mouseDown:(BOOL)fp36
{
	if ([Chax boolForKey:@"UseCustomContactListFonts"]) {
		NSData *data = [Chax objectForKey:@"ContactListGroupFont"];
		if (data) {
			NSFont *font = [NSUnarchiver unarchiveObjectWithData:data];
			if (font) {
				NSMutableAttributedString *string = [[NSMutableAttributedString alloc] initWithAttributedString:fp24];
				[string addAttribute:NSFontAttributeName value:font range:NSMakeRange(0, [[fp24 string] length])];
				[super _drawGroupRowInRect:fp8 withTitle:string andNumber:fp28 mouseOver:fp32 mouseDown:fp36];
				[string release];
				return;
			}
		}
	}
	
	[super _drawGroupRowInRect:fp8 withTitle:fp24 andNumber:fp28 mouseOver:fp32 mouseDown:fp36];
}

- (void)chax_popToFront
{
	[[self window] setLevel:NSStatusWindowLevel];
	[[self window] setLevel:NSNormalWindowLevel];
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
	[HaxAnimatingTableView cancelPreviousPerformRequestsWithTarget:self selector:@selector(chax_popToFront) object:nil];
	[super draggingExited:sender];
}

@end
