/*
 * HaxBuddyPicture.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "HaxBuddyPicture.h"
#import "Chax.h"

static NSMutableDictionary *_reps = nil;
static NSMutableDictionary *_timers = nil;

@implementation HaxBuddyPicture

- (void)drawInRect:(struct _NSRect)fp8 inView:(id)fp24 operation:(int)fp28 fraction:(float)fp32
{
	if ([fp24 isKindOfClass:[AnimatingTableView class]] && [Chax boolForKey:@"AnimateIcons"] && _data != nil) {
		NSArray *reps = [NSBitmapImageRep imageRepsWithData:_data];
		
		if ([reps count] > 0) {
			NSBitmapImageRep *rep = [reps objectAtIndex:0];
			int frames = [[rep valueForProperty:NSImageFrameCount] intValue];
		
			if (frames > 1) {
				
				int currentFrame = 0;
				
				if (_reps == nil) {
					_reps = [[NSMutableDictionary alloc] init];
					_timers = [[NSMutableDictionary alloc] init];
				}
				
				if ([_reps objectForKey:[self owner]]) {
					currentFrame = [[_reps objectForKey:[self owner]] intValue];
				}
				
				[rep setProperty:NSImageCurrentFrame withValue:[NSNumber numberWithInt:currentFrame]];
				
				NSImage *image = [[NSImage alloc] init];
				[image addRepresentation:rep];
				[image setFlipped:[fp24 isFlipped]];
				[fp24 lockFocus];
				[image drawInRect:fp8 fromRect:NSMakeRect(0, 0, [rep size].width, [rep size].height) operation:fp28 fraction:fp32];
				[fp24 unlockFocus];
				[image release];
				
				if ([_timers objectForKey:[self owner]] == nil) {
					currentFrame = (currentFrame + 1) % frames;
					[_timers setObject:[NSTimer scheduledTimerWithTimeInterval:[[rep valueForProperty:NSImageCurrentFrameDuration] floatValue] target:self selector:@selector(reloadTableItem:) userInfo:[NSDictionary dictionaryWithObjectsAndKeys:fp24, @"View", [NSNumber numberWithInt:[fp24 rowAtPoint:fp8.origin]], @"Row", [NSNumber numberWithInt:currentFrame], @"Frame", nil, nil] repeats:NO] forKey:[self owner]];
				}
				return;
			}
		}
	}
	[super drawInRect:fp8 inView:fp24 operation:fp28 fraction:fp32];
}

- (BOOL)animated
{
	return [_reps objectForKey:[self owner]] != nil;
}

- (void)reloadTableItem:(NSTimer *)timer
{
	NSDictionary *info = [timer userInfo];
	AnimatingTableView *tableView = [info objectForKey:@"View"];
	int row = [[info objectForKey:@"Row"] intValue];
	
	[_timers removeObjectForKey:[self owner]];
	[_reps setObject:[info objectForKey:@"Frame"] forKey:[self owner]];
	if ([[[tableView dataSource] presentityAtRow:row] isEqual:[self owner]]) {
		[tableView setRowNeedsDisplay:row];
	} else {
		[tableView reloadData];
	}
}

@end
