/*
 * HaxChatDoc.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "HaxChatDoc.h"
#import <AddressBook/AddressBook.h>
#import "Chax.h"
#import "DockIconController.h"
#import "HaxChatController.h"
#import "TabbedChatController.h"

@implementation HaxChatDoc

+ (id)chatDocComposingWithPresentities:(id)fp8 style:(int)fp12
{
	//Check to make sure there aren't any existing composing views in tabs already
	NSEnumerator *docEnumerator = [[[TabbedChatController sharedController] chatDocs] objectEnumerator];
	ChatDoc *nextItem;
	
	while ( (nextItem = [docEnumerator nextObject]) ) {
		if ([[[nextItem chat] otherPeople] isEqualToArray:fp8]) {
			[[TabbedChatController sharedController] showChatController:[nextItem controller]];
			return nextItem;
		}
	}
	return [super chatDocComposingWithPresentities:fp8 style:fp12];
}

- (void)close
{
	if ([[TabbedChatController sharedController] containsChatController:[self controller]]) {
		[[TabbedChatController sharedController] removeChatController:[self controller]];
		return;
	}
	
	[[DockIconController sharedController] removeSender:[[[self chat] otherPerson] name]];
	
	//Save the window frame information
	if ([Chax boolForKey:@"SaveWindowFrames"]) {
		NSMutableDictionary *presentities = [[Chax objectForKey:@"CustomPresentities"] mutableCopy];
		NSString *key = [[[[self chat] otherPerson] abPerson] uniqueId], *userID = [[[self chat] otherPerson] ID], *frameString = [[[self controller] window] stringWithSavedFrame];
		NSDictionary *info;
		NSMutableDictionary *mutableInfo;
		
		if (presentities == nil) {
			presentities = [[NSMutableDictionary alloc] init];
		}
		
		if (key == nil) {
			key = userID;
		}
		info = [presentities objectForKey:key];
		
		if (userID) {
			if (info) {
				mutableInfo = [info mutableCopy];
				[mutableInfo setObject:frameString forKey:[@"WindowFrame." stringByAppendingString:userID]];
				[presentities setObject:mutableInfo forKey:key];
				[mutableInfo release];
			} else {
				[presentities setObject:[NSDictionary dictionaryWithObject:frameString forKey:[@"WindowFrame." stringByAppendingString:userID]] forKey:key];
			}
			[Chax setObject:presentities forKey:@"CustomPresentities"];
		}
		[presentities release];
	}
	
	if (![(HaxChatController *)_controller isTab] && [_controller retainCount] > 1) {
		[_controller release];
	}
	
	[super close];
}

- (id)displayName
{
	NSDictionary *dict = [[self chat] metadataDictionary];
	if ([[dict objectForKey:@"ChatRoom"] length] > 0) {
		return [dict objectForKey:@"ChatRoom"];
	}
	return [super displayName];
}

- (BOOL)loadDataRepresentation:(id)fp8 ofType:(id)fp12
{
	BOOL value = [super loadDataRepresentation:fp8 ofType:fp12];
	[[self chat] setChatDoc:self];
	return value;
}

- (NSWindow *)windowForSheet
{
	return [[self controller] isTab] ? [[TabbedChatController sharedController] windowForChatController:[self controller]] : [super windowForSheet];
}

@end
