/*
 * HaxPeopleList.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "HaxPeopleList.h"
#import "HaxChatController.h"
#import "Hax_Presentity.h"
#import "Chax.h"

@implementation HaxPeopleList

/*- (void)_recomputePresentityInItem:(id)fp8 isVisible:(BOOL)fp12 withOtherPresentity:(id)fp16
{
	NSArray *presentities = [[fp8 presentity] presentities];
	int i;
	for (i = 0; i < [presentities count]; i++) {
		Presentity *presentity = [presentities objectAtIndex:i];
		PeopleListItem *item = [[PeopleListItem alloc] initAsPresentity:presentity inGroup:nil];
		NSLog(@"Adding presentity: %@", presentity);
		if ([presentity status] > 1 && ![[presentity ID] isEqualToString:[[fp8 presentity] ID]] && ![_items containsObject:item]) {
			[_items addObject:[item autorelease]];
			//[presentity setBestPresentity:presentity];
		} else {
			[item release];
		}
	}
	[super _recomputePresentityInItem:fp8 isVisible:fp12 withOtherPresentity:fp16];
}*/

- (void)calculateTableHeight
{
	//This means showing of buddy pictures is being turned off
	if (![Chax boolForKey:@"SkipCustomContactListHeight"] && [Chax boolForKey:@"UseCustomContactListFonts"] && (_viewOptions & 1 == 1)) {
		NSData *data = [Chax dataForKey:@"ContactListFont"];
		NSFont *font = (data != nil) ? [NSUnarchiver unarchiveObjectWithData:data] : nil;
		
		if (font) {
			[_table setRowHeight:(int)([font xHeight] + [font ascender] - [font descender])];
		}
	} else {
		float height = (_viewOptions & 1 == 1) ? 20.0 : 32.0;
		[_table setRowHeight:height];
	}
}

- (void)endChanges
{
	[super endChanges];
	
	if ([[self delegate] isKindOfClass:[PeopleListController class]] && [Chax boolForKey:@"AutoresizeContactList"] && ![[[self delegate] service] justLoggedIn]) {
		[[[self delegate] class] cancelPreviousPerformRequestsWithTarget:[self delegate] selector:@selector(resizeWindow) object:nil];
		[[self delegate] performSelector:@selector(resizeWindow) withObject:nil afterDelay:0.0];
	}
}

- (void)tableView:(id)fp8 returnKeyPressedAtRow:(int)fp12;
{
	if ([Chax boolForKey:@"WarnMobile"]) {
		Presentity *presentity = [self presentityAtRow:fp12];
		if ([presentity mobileDeviceName] != nil && [ChatDoc existingChatDocWithPresentities:[NSArray arrayWithObject:presentity]] == nil) {
			//Run a mobile user warning
			NSString *name = [presentity firstName];
			
			if (name == nil || [name length] == 0) {
				name = [presentity name];
			}
			
			if (NSRunInformationalAlertPanel(LocalizedString(@"mobile_title"), LocalizedString(@"mobile_msg"), nil, @"Cancel", nil, [presentity name], name) == NSAlertAlternateReturn) {
				return;
			}
		}
	}
	[super tableView:fp8 returnKeyPressedAtRow:fp12];
}

- (void)tableView:(id)fp8 groupRowClicked:(int)fp12
{
	[super tableView:fp8 groupRowClicked:fp12];
	
	if ([[self delegate] isKindOfClass:[PeopleListController class]] && [Chax boolForKey:@"AutoresizeContactList"] && ![[[self delegate] service] justLoggedIn]) {
		[[[self delegate] class] cancelPreviousPerformRequestsWithTarget:[self delegate] selector:@selector(resizeWindow) object:nil];
		[[self delegate] performSelector:@selector(resizeWindow) withObject:nil afterDelay:0.0];
	}
}

- (NSDragOperation)tableView:(NSTableView *)tableView validateDrop:(id <NSDraggingInfo>)info proposedRow:(int)row proposedDropOperation:(NSTableViewDropOperation)operation
{
	if ([tableView respondsToSelector:@selector(chax_popToFront)]) {
		[tableView performSelector:@selector(chax_popToFront) withObject:nil afterDelay:1.0];
	}
	[super tableView:tableView validateDrop:info proposedRow:row proposedDropOperation:operation];
}

@end
