/*
 * HaxPeopleListController.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "HaxPeopleListController.h"
#import "Chax.h"
#import "TabbedChatController.h"
#import "HaxPeopleList.h"
#import "HaxChatController.h"
#import <Carbon/Carbon.h>

static NSString *_previousMessage = nil;
static BOOL _screensaverAwayed = NO, _resizeAllowed = NO;

@implementation HaxPeopleListController

- (id)init
{
	if ( (self = [super init]) ) {
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:@"ReloadContactList" object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSApplicationWillResignActiveNotification object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSApplicationDidBecomeActiveNotification object:nil];
		
		[[NSDistributedNotificationCenter defaultCenter] addObserver:self selector:@selector(screensaverNotificationReceived:) name:@"com.apple.screensaver.didstart" object:nil];
		[[NSDistributedNotificationCenter defaultCenter] addObserver:self selector:@selector(screensaverNotificationReceived:) name:@"com.apple.screensaver.didstop" object:nil];
	}
	return self;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}

- (BOOL)_addVCButtonIsVideo:(BOOL)fp8
{
	if ([Chax boolForKey:@"HideMyAVStatus"]) {
		return NO;
	}
	return [super _addVCButtonIsVideo:fp8];
}

- (void)_serviceNoLongerJustLoggedIn
{
	[super _serviceNoLongerJustLoggedIn];
	[self performSelector:@selector(resizeWindow) withObject:nil afterDelay:0.5];
}

- (void)resizeWindow
{
	if ([_myStatusController currentStatus] <= 1) {
		_resizeAllowed = NO;
		[HaxPeopleListController cancelPreviousPerformRequestsWithTarget:self selector:@selector(resizeAllowed) object:nil];
		return;
	}
	
	if ([Chax boolForKey:@"AutoresizeContactList"] && [_myStatusController currentStatus] > 1) {
		if (!_resizeAllowed) {
			[self performSelector:@selector(resizeAllowed) withObject:nil afterDelay:2.0];
			return;
		}
		
		if (![[self service] justLoggedIn]) {
			NSRect frame = [[self window] frame], screenFrame = [[[self window] screen] frame];
			NSRect preferredFrame = [self windowWillUseStandardFrame:[self window] defaultFrame:screenFrame];
			
			//if the bottom edge of the window is within 20 pixels of the bottom and the top of the buddy list isn't at the top of the screen, resize upwards if possible
			//otherwise resize down as usual
			if (frame.origin.y < 20 && (screenFrame.size.height - GetMBarHeight() - (frame.origin.y + frame.size.height) > 10)) {
				//we're within 30 pixels of the bottom of the screen, this means we should resize upward
				preferredFrame.origin.y = frame.origin.y;
			} else {
				preferredFrame.origin.y -= 3;
			}
			
			preferredFrame.size.width = [[self window] frame].size.width;
			preferredFrame.origin.x = [[self window] frame].origin.x;
			preferredFrame.size.height += 3;
			
			[[self window] setFrame:preferredFrame display:YES animate:YES];
		}
	}
}

- (void)resizeAllowed
{
	if ([_myStatusController currentStatus] > 1) {
		_resizeAllowed = YES;
		[self resizeWindow];
	}
}

- (void)notificationReceived:(NSNotification *)note
{
	if ([[note name] isEqualToString:@"ReloadContactList"]) {
		[(NSTableView *)[[self peopleList] table] reloadData];
		[[self peopleList] calculateTableHeight];
		[self resizeWindow];
	} else if ([Chax boolForKey:@"HideOnDeactivate"]) {
		if ([[note name] isEqualToString:NSApplicationWillResignActiveNotification]) {
			[NSApp hide:nil];
		} else {
			NSEnumerator *enumerator = [[NSApp orderedWindows] objectEnumerator];
			NSWindow *next;
			while ( (next = [enumerator nextObject]) ) {
				if ([next isVisible] && ([[next delegate] isKindOfClass:[ChatDoc class]] || [[next delegate] isKindOfClass:[TabbedChatController class]])) {
					[next orderFront:nil];
				}
			}
			
			if ([[[NSApp keyWindow] delegate] isKindOfClass:[ChatDoc class]]) {
				[[NSApp keyWindow] orderFront:nil];
			}
		}
	}
}

- (void)toggleHidePictures:(id)fp8
{
	[super toggleHidePictures:fp8];
	[self resizeWindow];
	[[self peopleList] performSelector:@selector(calculateTableHeight) withObject:nil afterDelay:0.0];
}

- (void)screensaverNotificationReceived:(NSNotification *)note
{
	if ([Chax boolForKey:@"AwayOnScreenSaver"] && [[note name] isEqualToString:@"com.apple.screensaver.didstart"]) {
		if ([Service myStatus] == 4 || [Service myStatus] == 2) {
			_previousMessage = [[Service myStatusMessage] retain];
			[Service setStatus:3 message:[Chax stringForKey:@"AwayOnScreenSaverMessage"]];
			_screensaverAwayed = YES;
		}
	} else if (_screensaverAwayed && [Chax boolForKey:@"UnawayOnScreenSaver"] && [[note name] isEqualToString:@"com.apple.screensaver.didstop"]) {
		if ([Service myStatus] == 3) {
			[Service setStatus:4 message:_previousMessage];
			[_previousMessage release];
		}
		_screensaverAwayed = NO;
	}
}

- (void)toggleTextStatus:(id <NSMenuItem>)sender
{
	[sender setState:(([sender state] == NSOnState) ? NSOffState : NSOnState)];
	[Chax setBool:(([sender state] == NSOnState) ? NO : YES) forKey:[NSString stringWithFormat:@"%@.HideTextStatus", [self className]]];
	[(NSTableView *)[[self peopleList] table] reloadData];
}

- (void)personDoubleClicked:(id)sender
{
	if ([Chax boolForKey:@"WarnMobile"]) {
		Presentity *presentity = [[self peopleList] presentityAtRow:[sender clickedRow]];
		if ([presentity mobileDeviceName] != nil && [ChatDoc existingChatDocWithPresentities:[NSArray arrayWithObject:presentity]] == nil) {
			//Run a mobile user warning
			NSString *name = [presentity firstName];
			
			if (name == nil || [name length] == 0) {
				name = [presentity name];
			}
			
			if (NSRunInformationalAlertPanel(LocalizedString(@"mobile_title"), LocalizedString(@"mobile_msg"), nil, @"Cancel", nil, [presentity name], name) == NSAlertAlternateReturn) {
				return;
			}
		}
	}
	[super personDoubleClicked:sender];
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)sender
{
	if ([sender tag] == 13371) {
		[sender setState:([Chax boolForKey:[NSString stringWithFormat:@"%@.HideTextStatus", [self className]]] ? NSOffState : NSOnState)];
	}
	return [super validateMenuItem:sender];
}

@end
