/*
 * HaxPeopleListItem.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import <AddressBook/AddressBook.h>
#import "HaxPeopleListItem.h"
#import "Chax.h"

@implementation HaxPeopleListItem

- (id)displayName
{
	NSMutableAttributedString *name = [super displayName];
	
	/*if ([[[self presentity] presentities] count] > 1) {
		ABMultiValue *values = [[[self presentity] abPerson] valueForProperty:kABAIMInstantProperty];
		int i;
		for (i = 0; i < [values count]; i++) {
			if ([[values valueAtIndex:i] isEqualToString:[[self presentity] ID]]) {
				NSString *label = [values labelAtIndex:i];
				[name appendAttributedString:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@" (%@)", [label substringWithRange:NSMakeRange(4, [label length] - 8)]]] autorelease]];
			}
		}
	}*/
	
	if ([Chax boolForKey:@"UseCustomContactListFonts"]) {
		NSData *data = [Chax objectForKey:@"ContactListFont"];
		if (data) {
			NSFont *font = [NSUnarchiver unarchiveObjectWithData:data];
			if (font) {
				[name addAttribute:NSFontAttributeName value:font range:NSMakeRange(0, [[name string] length])];
			}
		}
	}
	
	return name;
}

- (int)_comparePresentitiesWith:(id)fp8 withSortParameters:(CDAnonymousStruct8 *)fp12
{
	if ([Chax boolForKey:@"UseNicknames"]) {
		Presentity *presentity = [fp8 presentity], *selfPresentity = [self presentity];
		
		//run the standard compare if the statuses are differnt and _field1[0] is 3
		if ([presentity status] != [selfPresentity status] && fp12->_field1[0] == 3) {
			return [super _comparePresentitiesWith:fp8 withSortParameters:fp12];
		} else {
			//sort by nickname if there is one
			NSString *nickname = [presentity ID], *selfNickname = [selfPresentity ID];
			
			if ([presentity abPerson]) {
				if ( !(nickname = [[presentity abPerson] valueForProperty:kABNicknameProperty]) ) {
					if (fp12->_field1[0] == 3) {
						nickname = (fp12->_field1[1] == 1) ? [presentity firstName] : [presentity lastName];
					} else {
						nickname = (fp12->_field1[0] == 1) ? [presentity firstName] : [presentity lastName];
					}
				}
			}
			
			if ([selfPresentity abPerson]) {
				if ( !(selfNickname = [[selfPresentity abPerson] valueForProperty:kABNicknameProperty]) ) {
					if (fp12->_field1[0] == 3) {
						selfNickname = (fp12->_field1[1] == 1) ? [selfPresentity firstName] : [selfPresentity lastName];
					} else {
						selfNickname = (fp12->_field1[0] == 1) ? [selfPresentity firstName] : [selfPresentity lastName];
					}
				}
			}
			
			return [selfNickname caseInsensitiveCompare:nickname];
		}
	} else {
		return [super _comparePresentitiesWith:fp8 withSortParameters:fp12];
	}
}

@end
