/*
 * HaxSecureWindow.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import <objc/objc-class.h>
#import "HaxSecureWindow.h"
#import "HaxChatController.h"
#import "Chax.h"
#import "TabbedChatController.h"

static BOOL forceFocus = NO;

@implementation HaxSecureWindow

+ (void)forceFocus
{
	forceFocus = YES;
}

- (void)endForceFocus
{
	forceFocus = NO;
}

- (void)makeKeyAndOrderFront:(id)sender
{
	if ([self delegate] && [[[self delegate] class] isEqual:[ChatDoc class]] && [[[self delegate] controller] respondsToSelector:@selector(isLoaded)]) {
		//Stop windows from displaying and show the tab window instead if enabled
		if ([[[self delegate] controller] isTab]) {
			if ([[TabbedChatController sharedController] containsChatController:[[self delegate] controller]]) {
				[[TabbedChatController sharedController] showChatController:[[self delegate] controller]];
			}
			return;
		}
		
		if (![self isVisible] && [Chax boolForKey:@"SaveWindowFrames"]) {
			//Read saved presentity preferences and check to see if there is a saved frame to set to
			NSDictionary *presentities = [Chax objectForKey:@"CustomPresentities"], *info;
			NSString *key = [[[[[self delegate] chat] otherPerson] abPerson] uniqueId], *userID = [[[[self delegate] chat] otherPerson] ID];
			if (key == nil) {
				key = userID;
			}
			
			info = [presentities objectForKey:key];
			if (info) {
				[self setFrameFromString:[info objectForKey:[@"WindowFrame." stringByAppendingString:userID]]];
			}
		}
		
		if ([Chax boolForKey:@"SkipNewMessageNotification"]) {
			if ([[[self delegate] controller] isLoaded]) {
				if (forceFocus) {
					[super makeKeyAndOrderFront:nil];
					[self performSelector:@selector(endForceFocus) withObject:nil afterDelay:0.5];
					return;
				}
				
				if ([NSApp isActive] && ![[[self delegate] controller] isComposing] && [[[[self delegate] chat] dateCreated] timeIntervalSinceNow] > -4) {
					if (![Chax boolForKey:@"OrderBackNewMessages"]) {
						[super orderFront:nil];
					} else {
						NSWindow *lastFront = [NSApp mainWindow];
						if ([[[lastFront delegate] class] isEqual:[ChatDoc class]]) {
							[lastFront performSelector:@selector(makeKeyAndOrderFrontIfVisible:) withObject:self afterDelay:2.0];
						} else {
							[super makeKeyAndOrderFront:nil];
						}
					}
				} else {
					[[[[self delegate] controller] transcriptView] restoreDefaultBackground];
					[super makeKeyAndOrderFront:nil];
				}
			}
		} else {
			[super makeKeyAndOrderFront:nil];
		}
	} else {
		[super makeKeyAndOrderFront:nil];
	}
}

- (void)orderFront:(id)sender
{
	if ([self delegate] && [Chax boolForKey:@"EnableTabs"] && [Chax boolForKey:@"SkipNewMessageNotification"]) {
		if ([[[self delegate] class] isEqual:[StagedChatNotifier class]]) {
			id var;
			object_getInstanceVariable([self delegate], "_chat", (void **)&var);
			if (var == nil) {
				[super orderFront:sender];
			} else {
				return;
			}
		} else if ([[[self delegate] class] isEqual:[ChatDoc class]]) {
			return;
		}
	}
	[super orderFront:sender];
}

- (void)makeKeyAndOrderFrontIfVisible:(id)sender
{
	if ([self isVisible]) {
		[self makeKeyAndOrderFront:sender];
	}
}

- (void)toggleIsOnTop:(id)sender
{
	[self setLevel:([self level] == NSNormalWindowLevel) ? NSStatusWindowLevel : NSNormalWindowLevel];
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)sender
{
	if ([self delegate] && ([[[self delegate] class] isEqual:[ChatDoc class]] || [[[self delegate] class] isEqual:[VideoChatController class]] || [[[self delegate] class] isEqual:[AudioChatController class]]) && [sender tag] == 13374) {
		[sender setState:([self level] == NSNormalWindowLevel) ? NSOffState : NSOnState];
		return YES;
	}
	return [super validateMenuItem:sender];
}

@end
