/*
 * HaxTabView.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "HaxTabView.h"
#import "HaxTabViewItem.h"
#import "iChat.h"

@implementation HaxTabView

- (void)awakeFromNib
{
	_draggedTab = nil;
	_dragInsertIndex = -1;
}

- (void)moveTab:(NSTabViewItem *)tab toIndex:(int)index
{
	int oldIndex = [self indexOfTabViewItem:tab];
	if ((oldIndex != NSNotFound) && (oldIndex != index)) {
		[tab retain];
		[self removeTabViewItem:tab];
		[self insertTabViewItem:tab atIndex:index];
		[self selectTabViewItem:tab];
		[tab release];
	}
}

- (BOOL)isReordering
{
	return _draggedTab != nil;
}

- (void)mouseUp:(NSEvent *)event {
	if (_draggedTab) {
		NSPoint releasedPoint = [self convertPoint:[event locationInWindow] fromView:[[self window] contentView]];
		HaxTabViewItem *releasedTab = (HaxTabViewItem *)[self tabViewItemAtPoint:releasedPoint];
		if (releasedTab && ![releasedTab isEqualTo:_draggedTab]) {
			int status = [_draggedTab chatStatus];
			[self moveTab:_draggedTab toIndex:[self indexOfTabViewItem:releasedTab]];
			[_draggedTab setChatStatus:status];
		}
		_draggedTab = nil;
	}
	[super mouseUp:event];
}

- (void)mouseDragged:(NSEvent *)event {
	if (_draggedTab == nil) {
		NSPoint clickedPoint;
		clickedPoint = [self convertPoint:[event locationInWindow] fromView:[[self window] contentView]];
		_draggedTab = (HaxTabViewItem *)[self tabViewItemAtPoint:clickedPoint];
	}
	
	if (_draggedTab) {
		NSPoint currentPoint = [self convertPoint:[event locationInWindow] fromView:[[self window] contentView]];
		HaxTabViewItem *curTab = (HaxTabViewItem *)[self tabViewItemAtPoint:currentPoint];
		if (curTab && ![curTab isEqualTo:_draggedTab]) {
			int status = [_draggedTab chatStatus];
			[self moveTab:_draggedTab toIndex:[self indexOfTabViewItem:curTab]];
			[self selectTabViewItem:_draggedTab];
			[_draggedTab setChatStatus:status];
		}
	}
	[super mouseDragged:event];
}

- (void)drawRect:(NSRect)rect
{
	[super drawRect:rect];
}

@end
