/*
 * Hax_ActiveChat.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "MethodSwizzle.h"
#import "iChat.h"
#import "Chax.h"
#import "DockIconController.h"
#import "TabbedChatController.h"
#import "Hax_Presentity.h"
#import "HaxAIMLGenerator.h"
#import "HaxChatController.h"

static BOOL _postingAway = NO;

@implementation ActiveChat (Hax_hax_hax)

+ (void)load
{
	MethodSwizzle([self class], @selector(_personStatusChanged:), @selector(hax_personStatusChanged:));
	MethodSwizzle([self class], @selector(_sendAutoReply), @selector(hax_sendAutoReply));
	MethodSwizzle([self class], @selector(writeToFile:ofType:), @selector(hax_writeToFile:ofType:));
}

- (void)hax_personStatusChanged:(NSNotification *)note
{
	Presentity *pres = [note object];
	int status = [pres status], prev = [pres previousStatus];
	
	if ([[[self chatDoc] controller] isTab]) {
		if (status == 1) {
			[[TabbedChatController sharedController] setStatus:3 ofChat:self];
		} else if (status > 1 && prev == 1) {
			[[TabbedChatController sharedController] setStatus:0 ofChat:self];
		}
	}
	
	if ([Chax boolForKey:@"ShowStatusChanges"]) {
		if (status == 3) {
			if (!_postingAway && (prev != 3) && ![Chax boolForKey:@"ShowStatusChanges.AwayOff"]) {
				[self performSelector:@selector(addAwayAnnouncement:) withObject:pres afterDelay:1.0];
				_postingAway = YES;
			} else if (!_postingAway && (prev == 3) && ![Chax boolForKey:@"ShowStatusChanges.AwayChangeOff"]) {
				[self performSelector:@selector(addAwayChangeAnnouncement:) withObject:pres afterDelay:1.0];
				_postingAway = YES;
			}
		} else if (status == 2 && ![Chax boolForKey:@"ShowStatusChanges.IdleOff"]) {
			[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"idle_status"), [pres name]]];
		} else if (status == 4 && prev != 1) {
			if (prev == 1 && [Chax boolForKey:@"ShowStatusChanges.OnlineOff"]) {
				//Do nothing, skip the "X is now online." message
				return;
			} else if (prev != 4 && ![Chax boolForKey:@"ShowStatusChanges.AvailableOff"]) {
				if ([[pres statusMessage] length] == 0) {
					[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"available_status"), [pres name]]];
				} else {
					[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"available_status_msg"), [pres name], [pres statusMessage]]];
				}
			} else if ([[pres statusMessage] length]) {
				//Here we have to decide to either show the iTunes song change message or the status change method
				//If there is a statusURL, it is a song change message
				if ([pres hax_statusURL] != nil) {
					if (![Chax boolForKey:@"ShowStatusChanges.SongsOff"]) {
						[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"changed_song_msg"), [pres name], [pres statusMessage]]];
					}
				} else if (![Chax boolForKey:@"ShowStatusChanges.MessageChangeOff"]) {
					[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"changed_status_msg"), [pres name], [pres statusMessage]]];
				}
			}
		} else if (status == 1 && [Chax boolForKey:@"ShowStatusChanges.OfflineOff"]) {
			//Do nothing, skip the "X is now offline." message
			return;
		}
	}
	[self hax_personStatusChanged:note];
}

- (void)hax_sendAutoReply
{
	//If we're logged in through ICQ and we're talking to an ICQ user flag the AIML generator to send plain text
	if ([[[Me presentityForService:[self service]] ID] isICQorPhone] && [[[self otherPerson] ID] isICQorPhone] && ![Chax boolForKey:@"DisablePlainText"]) {
		[HaxAIMLGenerator nextMessagePlainText];
	}
	
	if ([Chax boolForKey:@"CustomAutoAway"]) {
		NSString *message = [Chax stringForKey:@"CustomAutoAwayMessage"];
		//Send our custom away message
		if (message != nil) {
			[self sendMessage:[InstantMessage fromMeOnService:[self service] withText:[[[NSMutableAttributedString alloc] initWithString:message] autorelease] outgoingFile:nil flags:45]];
			
			if (![Chax boolForKey:@"AutoReplyOnce"]) {
				[NSTimer scheduledTimerWithTimeInterval:180 target:self selector:@selector(clearCustomAutoAway) userInfo:nil repeats:NO];
			}
			_needsToSendAutoReply = NO;
			_alreadyReplied = YES;
		}
	} else {
		[self hax_sendAutoReply];
		
		if ([Chax boolForKey:@"AutoReplyOnce"]) {
			[_autoreplyTimer invalidate];
			[_autoreplyTimer release];
			_autoreplyTimer = nil;
		}
	}
}

- (void)clearCustomAutoAway
{
	_alreadyReplied = NO;
	[self _clearAutorepliedFlag];
	[self setCanAutoReply:YES];;
}

- (void)addAwayAnnouncement:(id)sender
{
	if ([[sender statusMessage] length]) {
		[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"away_status_msg"), [sender name], [sender statusMessage]]];
	} else {
		[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"away_status"), [sender name]]];
	}
	_postingAway = NO;
}

- (void)addAwayChangeAnnouncement:(id)sender
{
	if ([[sender statusMessage] length]) {
		[self addAnnouncementString:[NSString stringWithFormat:LocalizedString(@"away_status_change"), [sender name], [sender statusMessage]]];
	}
	_postingAway = NO;
}

- (BOOL)hax_writeToFile:(id)fp8 ofType:(id)fp12
{
	[self hax_writeToFile:fp8 ofType:fp12];
	[self performSelector:@selector(writePostaction:) withObject:fp8 afterDelay:1.0];
}

- (void)writePostaction:(id)sender
{
	if ([Chax boolForKey:@"ActiveChat.UseProxyIcons"]) {
		[[[[self chatDoc] controller] window] setRepresentedFilename:sender];
	}
}

@end
