/*
 * Hax_Fezz.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "MethodSwizzle.h"
#import "iChat.h"
#import "Chax.h"
#import "TabbedChatController.h"
#import "HaxChatController.h"

static BOOL _shouldReopen = NO;

@implementation Fezz (Hax_hax_hax)

+ (void)load
{
	MethodSwizzle([self class], @selector(hax_applicationShouldTerminate:), @selector(applicationShouldTerminate:));
	MethodSwizzle([self class], @selector(hax_applicationShouldHandleReopen:hasVisibleWindows:), @selector(applicationShouldHandleReopen:hasVisibleWindows:));
	MethodSwizzle([self class], @selector(hax_applicationDidFinishLaunching:), @selector(applicationDidFinishLaunching:));
}

- (void)hax_setReopenFlag
{
	//This gets around having the custom contact list opening behavior from taking effect on launch
	_shouldReopen = YES;
}

- (void)hax_applicationDidFinishLaunching:(id)fp8
{
	[self performSelector:@selector(hax_setReopenFlag) withObject:nil afterDelay:2.0];
	[self hax_applicationDidFinishLaunching:fp8];
}

- (BOOL)hax_applicationShouldHandleReopen:(id)fp8 hasVisibleWindows:(BOOL)fp12
{
	if (!_shouldReopen) {
		return [self hax_applicationShouldHandleReopen:fp8 hasVisibleWindows:fp12];
	}
	
	BOOL hasKeyWindow = ([NSApp keyWindow] != nil);
	NSNumber *aim, *jabber, *bonjour;
	
	aim = [Chax objectForKey:@"ShowAIMOnReactivate"];
	jabber = [Chax objectForKey:@"ShowJabberOnReactivate"];
	bonjour = [Chax objectForKey:@"ShowBonjourOnReactivate"];
	
	if (aim == nil && jabber == nil && bonjour == nil) {
		//this will cause iChat to display the highest priority contact list if none are currently open (priority is AIM, Bonjour, Jabber)
		return [self hax_applicationShouldHandleReopen:fp8 hasVisibleWindows:fp12];
	}
	
	if ([Chax boolForKey:@"ShowAIMOnReactivate"]) {
		if (hasKeyWindow) {
			[[BuddyList sharedInstance] displayWithoutKey];
		} else {
			[[BuddyList sharedInstance] display];
			hasKeyWindow = YES;
		}
	}
	
	if ([Chax boolForKey:@"ShowJabberOnReactivate"]) {
		if (hasKeyWindow) {
			[[JabberViewer sharedInstance] displayWithoutKey];
		} else {
			[[JabberViewer sharedInstance] display];
			hasKeyWindow = YES;
		}
	}
	
	if ([Chax boolForKey:@"ShowBonjourOnReactivate"]) {
		if (hasKeyWindow) {
			[[SubNetViewer sharedInstance] displayWithoutKey];
		} else {
			[[SubNetViewer sharedInstance] display];
		}
	}
	
	return NO;
}

- (NSApplicationTerminateReply)hax_applicationShouldTerminate:(NSApplication *)sender
{
	int reply;
	if ([Chax boolForKey:@"ConfirmQuit"] && [[NSApp orderedDocuments] count] > 0) {
		NSAlert *alert = [[NSAlert alloc] init];
		[alert setMessageText:NSLocalizedString(@"There are active chats. Are you sure you want to quit?", @"There are active chats. Are you sure you want to quit?")];
		[alert addButtonWithTitle:NSLocalizedString(@"Yes", @"Yes")];
		[[alert addButtonWithTitle:NSLocalizedString(@"No", @"No")] setKeyEquivalent:@"\e"];
		
		reply = ([alert runModal] == NSAlertFirstButtonReturn) ? NSTerminateNow : NSTerminateCancel;
		[alert release];
	} else {
		reply = [self hax_applicationShouldTerminate:sender];
	}
	return reply;
}

@end
