/*
 * Hax_Preferences.h
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import <Cocoa/Cocoa.h>
#import "NSPreferences.h"

@class Hax_PrefsView;

@interface Hax_Preferences : NSPreferencesModule
{
	IBOutlet NSButton *_aimListCheckbox;
	IBOutlet NSButton *_animateIconsCheckbox;
	IBOutlet NSButton *_autoAcceptFilesCheckbox;
	IBOutlet NSTextField *_autoAwayTextField;
	IBOutlet NSButton *_autoReconnectCheckbox;
	IBOutlet NSButton *_autoReplyOnceCheckbox;
	IBOutlet NSButton *_autoresizeContactListCheckbox;
	IBOutlet NSButton *_autoUpdateCheckbox;
	IBOutlet NSButton *_awayOnScreenSaverCheckbox;
	IBOutlet NSTextField *_awayMessageOnScreenSaverTextField;
	IBOutlet NSButton *_bonjourListCheckbox;
    IBOutlet NSTextField *_contactFontTextField;
	IBOutlet NSButton *_confirmQuitCheckbox;
	IBOutlet NSButton *_customAutoAwayCheckbox;
	IBOutlet NSTextField *_customAutoAwayTextField;
	IBOutlet NSButton *_customFontCheckbox;
	IBOutlet NSButton *_deminiaturizeMessagesCheckbox;
	IBOutlet NSButton *_disableAlertSoundsAvailableCheckbox;
	IBOutlet NSButton *_disableAlertSoundsAwayCheckbox;
	IBOutlet NSButton *_disableAlertSoundsIdleCheckbox;
	IBOutlet NSButton *_tabDisableOverflowMenuCheckbox;
	IBOutlet NSButton *_disablePIPCheckbox;
    IBOutlet NSColorWell *_dockBackgroundColorWell;
    IBOutlet NSMatrix *_dockNotificationTypeMatrix;
    IBOutlet NSColorWell *_dockTextColorWell;
	IBOutlet NSButton *_enableAutoAwayCheckbox;
	IBOutlet NSButton *_enableTabsCheckbox;
	IBOutlet NSWindow *_fileTransferOptions;
	IBOutlet NSButton *_flashBadgeCheckbox;
	IBOutlet NSTextField *_groupFontTextField;
	IBOutlet NSButton *_growlNotificationsCheckbox;
	IBOutlet NSButton *_hideMyAVStatusCheckbox;
	IBOutlet NSButton *_hideOnDeactivateCheckbox;
    IBOutlet NSButton *_iTMSLinksCheckbox;
	IBOutlet NSButton *_jabberListCheckbox;
	IBOutlet NSButton *_optionsButton;
	IBOutlet NSButton *_previewDockNotificationButton;
	IBOutlet NSButton *_saveFramesCheckbox;
	IBOutlet NSButton *_setContactFontButton;
	IBOutlet NSButton *_setGroupFontButton;
	IBOutlet NSButton *_setStatusMessageFontButton;
	IBOutlet NSButton *_showMenuBadgeCheckbox;
	IBOutlet NSButton *_showSmileyButtonCheckbox;
	IBOutlet NSButton *_showStatusChangesCheckbox;
	IBOutlet NSButton *_showUserIconsCheckbox;
    IBOutlet NSButton *_skipNotificationCheckbox;
	IBOutlet NSWindow *_statusChangeOptions;
	IBOutlet NSButton *_statusChangeAvailableCheckbox;
	IBOutlet NSButton *_statusChangeAwayCheckbox;
	IBOutlet NSButton *_statusChangeAway2Checkbox;
	IBOutlet NSButton *_statusChangeIdleCheckbox;
	IBOutlet NSButton *_statusChangeMessageCheckbox;
	IBOutlet NSButton *_statusChangeOfflineCheckbox;
	IBOutlet NSButton *_statusChangeOnlineCheckbox;
	IBOutlet NSButton *_statusChangeSongCheckbox;
	IBOutlet NSTextField *_statusMessageFontTextField;
	IBOutlet NSButton *_statusOptionsButton;
	IBOutlet NSPopUpButton *_tabBarLocationPopUpMenu;
	IBOutlet NSButton *_tabBringToFrontCheckbox;
	IBOutlet NSButton *_tabDisableCloseCheckbox;
	IBOutlet NSButton *_tabDisableTearOffCheckbox;
	IBOutlet NSButton *_tabHideTabBarCheckbox;
	IBOutlet NSPopUpButton *_tabKeyEquivalentPopUpMenu;
	IBOutlet NSButton *_tabSelectNewMessagesCheckbox;
	IBOutlet NSButton *_tabShowUserIconsCheckbox;
	IBOutlet NSColorWell *_tabUnreadMessagesColorWell;
	IBOutlet NSColorWell *_tabUserOfflineColorWell;
	IBOutlet NSColorWell *_tabUserTypingColorWell;
	IBOutlet NSButton *_transferAIMCheckbox;
	IBOutlet NSButton *_transferBonjourCheckbox;
	IBOutlet NSButton *_transferJabberCheckbox;
	IBOutlet NSPopUpButton *_transferGroupMenu;
	IBOutlet NSButton *_unawayOnScreenSaverCheckbox;
	IBOutlet NSButton *_unblockAVFileTransferCheckbox;
    IBOutlet NSButton *_useDockNotificationsCheckbox;
	IBOutlet NSButton *_useNicknamesCheckbox;
	IBOutlet NSButton *_usePreviewCheckbox;
    IBOutlet Hax_PrefsView *_view;
	IBOutlet NSButton *_warnMobileCheckbox;
	
	NSFont *_contactFont, *_statusMessageFont, *_groupFont;
	int _changingFont;
	NSImage *_icon;
}
+ (void)migratePreferences;

- (void)registerDefaults;

- (IBAction)changeFileTransferSetting:(id)sender;
- (IBAction)changeSetting:(id)sender;
- (IBAction)changeStatusChangeSetting:(id)sender;
- (IBAction)changeTabSetting:(id)sender;

- (void)setContactFont:(NSFont *)font;
- (NSFont *)contactFont;
@end
