/*
 * Hax_Preferences.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Chax.h"
#import "Hax_Preferences.h"
#import "Hax_PrefsView.h"
#import "HaxFezPreferences.h"
#import "HaxChatController.h"
#import "HaxChatInputLine.h"
#import "DockIconController.h"
#import "UpdateController.h"
#import "TabbedChatController.h"
#import <AddressBook/AddressBook.h>

#define SENDER_STATE ([(NSCell *)sender state] == NSOnState)
#define PREF_STATE(x) [Chax boolForKey:x] ? NSOnState : NSOffState
#define DONATE_URL [NSURL URLWithString:@"http://www.ksuther.com/chax/index.php?p=donate"]

static BOOL _AVRestartWarningShown = NO;

@implementation Hax_Preferences

+ (void)migratePreferences
{
	NSUserDefaults *df = [NSUserDefaults standardUserDefaults];
	[Chax setBool:[df boolForKey:@"AutoAcceptFiles"] forKey:@"AutoAcceptFiles"];
	[Chax setBool:[df boolForKey:@"AutoAcceptFiles.AIM"] forKey:@"AutoAcceptFiles.AIM"];
	[Chax setBool:[df boolForKey:@"AutoAcceptFiles.Bonjour"] forKey:@"AutoAcceptFiles.Bonjour"];
	[Chax setBool:[df boolForKey:@"AutoAcceptFiles.Jabber"] forKey:@"AutoAcceptFiles.Jabber"];
	[Chax setObject:[df stringForKey:@"AutoAcceptFiles.Group"] forKey:@"AutoAcceptFiles.Group"];
	[Chax setBool:[df boolForKey:@"AutoUpdateChax"] forKey:@"AutoUpdateChax"];
	[Chax setBool:[df boolForKey:@"BuddyList.HideTextStatus"] forKey:@"BuddyList.HideTextStatus"];
	[Chax setBool:[df boolForKey:@"ConfirmQuit"] forKey:@"ConfirmQuit"];
	[Chax setBool:[df boolForKey:@"CustomAutoAway"] forKey:@"CustomAutoAway"];
	[Chax setObject:[df stringForKey:@"CustomAutoAwayMessage"] forKey:@"CustomAutoAwayMessage"];
	[Chax setBool:[df boolForKey:@"BuddyList.HideTextStatus"] forKey:@"BuddyList.HideTextStatus"];
	[Chax setObject:[df dataForKey:@"DockBackgroundColor"] forKey:@"DockBackgroundColor"];
	[Chax setObject:[df dataForKey:@"DockForegroundColor"] forKey:@"DockForegroundColor"];
	[Chax setInteger:[df integerForKey:@"DockNotificationType"] forKey:@"DockNotificationType"];
	[Chax setBool:[df boolForKey:@"EnableDockNotification"] forKey:@"EnableDockNotification"];
	[Chax setBool:[df boolForKey:@"FlashBadge"] forKey:@"FlashBadge"];
	[Chax setBool:[df boolForKey:@"HideMusicStoreLinks"] forKey:@"HideMusicStoreLinks"];
	[Chax setBool:[df boolForKey:@"HideMyAVStatus"] forKey:@"HideMyAVStatus"];
	[Chax setBool:[df boolForKey:@"HideSmileyButton"] forKey:@"HideSmileyButton"];
	[Chax setBool:[df boolForKey:@"ShowStatusChanges"] forKey:@"ShowStatusChanges"];
	[Chax setBool:[df boolForKey:@"SkipNewMessageNotification"] forKey:@"SkipNewMessageNotification"];
	[Chax setBool:[df boolForKey:@"SubNetViewer.HideTextStatus"] forKey:@"SubNetViewer.HideTextStatus"];
	[Chax setBool:[df boolForKey:@"UnblockAVFileTransfers"] forKey:@"UnblockAVFileTransfers"];
	[Chax setBool:[df boolForKey:@"UseCustomContactListFonts"] forKey:@"UseCustomContactListFonts"];
	[Chax setBool:[df boolForKey:@"UseNicknames"] forKey:@"UseNicknames"];
	[Chax setBool:[df boolForKey:@"UsePreviewForImages"] forKey:@"UsePreviewForImages"];
	[Chax setBool:[df boolForKey:@"WarnMobile"] forKey:@"WarnMobile"];
	
	NSArray *fontArray = [df arrayForKey:@"ContactListTextStyle"];
	if ([fontArray count] == 2) {
		NSFont *font = [NSFont fontWithName:[fontArray objectAtIndex:0] size:[[fontArray objectAtIndex:1] floatValue]];
		if (font) {
			[Chax setObject:[NSArchiver archivedDataWithRootObject:font] forKey:@"ContactListFont"];
		}
	}
	
	fontArray = [df arrayForKey:@"ContactListStatusTextStyle"];
	if ([fontArray count] == 2) {
		NSFont *font = [NSFont fontWithName:[fontArray objectAtIndex:0] size:[[fontArray objectAtIndex:1] floatValue]];
		if (font) {
			[Chax setObject:[NSArchiver archivedDataWithRootObject:font] forKey:@"ContactListStatusFont"];
		}
	}
}

- (void)awakeFromNib
{
	//NSDictionary *tabs = [Chax dictionaryForKey:@"TabClasses"];
	BOOL useDockNotifications = [Chax boolForKey:@"EnableDockNotification"];
	NSString *awayMessage = [Chax stringForKey:@"CustomAutoAwayMessage"];
	
	if (![Chax objectForKey:@"DockForegroundColor"]) {
		[self registerDefaults];
	}
	
	//General tab
	[_customFontCheckbox setState:PREF_STATE(@"UseCustomContactListFonts")];
	[_setContactFontButton setEnabled:[Chax boolForKey:@"UseCustomContactListFonts"]];
	[_setStatusMessageFontButton setEnabled:[Chax boolForKey:@"UseCustomContactListFonts"]];
	[_setGroupFontButton setEnabled:[Chax boolForKey:@"UseCustomContactListFonts"]];
	[_iTMSLinksCheckbox setState:[Chax boolForKey:@"HideMusicStoreLinks"] ? NSOffState : NSOnState];
	[_useNicknamesCheckbox setState:PREF_STATE(@"UseNicknames")];
	[_animateIconsCheckbox setState:PREF_STATE(@"AnimateIcons")];
	[_autoresizeContactListCheckbox setState:PREF_STATE(@"AutoresizeContactList")];
	[_aimListCheckbox setState:PREF_STATE(@"ShowAIMOnReactivate")];
	[_jabberListCheckbox setState:PREF_STATE(@"ShowJabberOnReactivate")];
	[_bonjourListCheckbox setState:PREF_STATE(@"ShowBonjourOnReactivate")];
	[_autoReconnectCheckbox setState:PREF_STATE(@"AttemptAutoReconnect")];
	[_hideMyAVStatusCheckbox setState:PREF_STATE(@"HideMyAVStatus")];
	[_hideOnDeactivateCheckbox setState:PREF_STATE(@"HideOnDeactivate")];
	[_confirmQuitCheckbox setState:PREF_STATE(@"ConfirmQuit")];
	[_showMenuBadgeCheckbox setState:PREF_STATE(@"ShowMenuBadge")];
	
	//Away tab
	[_enableAutoAwayCheckbox setState:[AutoAway autoAwayEnabled] ? NSOnState : NSOffState];
	[_autoAwayTextField setDoubleValue:[AutoAway autoAwayDelay] / 60.0];
	[_autoAwayTextField setEnabled:([_enableAutoAwayCheckbox state] == NSOnState)];
	[_customAutoAwayCheckbox setState:PREF_STATE(@"CustomAutoAway")];
	[_customAutoAwayTextField setStringValue:(awayMessage == nil) ? @"" : awayMessage];
	[_customAutoAwayTextField setEnabled:[Chax boolForKey:@"CustomAutoAway"]];
	[_autoReplyOnceCheckbox setState:PREF_STATE(@"AutoReplyOnce")];
	[_awayOnScreenSaverCheckbox setState:PREF_STATE(@"AwayOnScreenSaver")];
	[_awayMessageOnScreenSaverTextField setStringValue:([Chax stringForKey:@"AwayOnScreenSaverMessage"] == nil) ? @"" : [Chax stringForKey:@"AwayOnScreenSaverMessage"]];
	[_awayMessageOnScreenSaverTextField setEnabled:[Chax boolForKey:@"AwayOnScreenSaver"]];
	[_unawayOnScreenSaverCheckbox setState:PREF_STATE(@"UnawayOnScreenSaver")];
	[_unawayOnScreenSaverCheckbox setEnabled:[Chax boolForKey:@"AwayOnScreenSaver"]];
	
	//Chats tab
	[_skipNotificationCheckbox setState:PREF_STATE(@"SkipNewMessageNotification")];
	[_showStatusChangesCheckbox setState:PREF_STATE(@"ShowStatusChanges")];
	[_statusOptionsButton setEnabled:[Chax boolForKey:@"ShowStatusChanges"]];
	[_showSmileyButtonCheckbox setState:[Chax boolForKey:@"HideSmileyButton"] ? NSOffState : NSOnState];
	[_usePreviewCheckbox setState:PREF_STATE(@"UsePreviewForImages")];
	[_warnMobileCheckbox setState:PREF_STATE(@"WarnMobile")];
	[_autoAcceptFilesCheckbox setState:PREF_STATE(@"AutoAcceptFiles")];
	[_optionsButton setEnabled:[Chax boolForKey:@"AutoAcceptFiles"]];
	[_unblockAVFileTransferCheckbox setState:PREF_STATE(@"UnblockAVFileTransfers")];
	[_disablePIPCheckbox setState:PREF_STATE(@"DisablePIP")];
	[_saveFramesCheckbox setState:PREF_STATE(@"SaveWindowFrames")];
	
	//Notifications tab
	[_useDockNotificationsCheckbox setState:useDockNotifications ? NSOnState : NSOffState];
	[_flashBadgeCheckbox setState:PREF_STATE(@"FlashBadge")];
	[_flashBadgeCheckbox setEnabled:useDockNotifications];
	[_dockNotificationTypeMatrix setEnabled:useDockNotifications];
	[_dockNotificationTypeMatrix selectCellAtRow:[Chax integerForKey:@"DockNotificationType"] column:0];
	[_dockTextColorWell setEnabled:useDockNotifications];
	[_dockBackgroundColorWell setEnabled:useDockNotifications];
	[_previewDockNotificationButton setEnabled:useDockNotifications];
	[_deminiaturizeMessagesCheckbox setState:PREF_STATE(@"DeminiaturizeOnNewMessages")];
	[_showUserIconsCheckbox setState:PREF_STATE(@"UseBuddyIconNotification")];
	[_growlNotificationsCheckbox setState:PREF_STATE(@"GrowlEnabled")];
	[_disableAlertSoundsAvailableCheckbox setState:PREF_STATE(@"DisableSoundWhenAvailable")];
	[_disableAlertSoundsAwayCheckbox setState:PREF_STATE(@"DisableSoundWhenAway")];
	[_disableAlertSoundsIdleCheckbox setState:PREF_STATE(@"DisableSoundWhenIdle")];
	
	[_dockTextColorWell setColor:[NSUnarchiver unarchiveObjectWithData:[Chax dataForKey:@"DockForegroundColor"]]];
	[_dockBackgroundColorWell setColor:[NSUnarchiver unarchiveObjectWithData:[Chax objectForKey:@"DockBackgroundColor"]]];
	
	//Tabs tab
	if ([Chax objectForKey:@"TabUnreadColor"] == nil) {
		[Chax setObject:[NSArchiver archivedDataWithRootObject:[NSColor redColor]] forKey:@"TabUnreadColor"];
		[Chax setObject:[NSArchiver archivedDataWithRootObject:[NSColor colorWithCalibratedRed:0.0 green:0.5 blue:0.0 alpha:1.0]] forKey:@"TabTypingColor"];
		[Chax setObject:[NSArchiver archivedDataWithRootObject:[NSColor darkGrayColor]] forKey:@"TabOfflineColor"];
	}
	
	BOOL enableTabs = [Chax boolForKey:@"EnableTabs"];
	[_enableTabsCheckbox setState:PREF_STATE(@"EnableTabs")];
	[_tabUnreadMessagesColorWell setColor:[NSUnarchiver unarchiveObjectWithData:[Chax dataForKey:@"TabUnreadColor"]]];
	[_tabUserTypingColorWell setColor:[NSUnarchiver unarchiveObjectWithData:[Chax dataForKey:@"TabTypingColor"]]];
	[_tabUserOfflineColorWell setColor:[NSUnarchiver unarchiveObjectWithData:[Chax dataForKey:@"TabOfflineColor"]]];
	[_tabBringToFrontCheckbox setState:PREF_STATE(@"TabFrontOnTextInvitation")];
	[_tabBringToFrontCheckbox setEnabled:PREF_STATE(@"EnableTabs")];
	[_tabSelectNewMessagesCheckbox setState:PREF_STATE(@"TabSelectNew")];
	[_tabSelectNewMessagesCheckbox setEnabled:PREF_STATE(@"EnableTabs") && PREF_STATE(@"TabFrontOnTextInvitation")];
	[_tabShowUserIconsCheckbox setState:PREF_STATE(@"TabShowUserIcons")];
	[_tabHideTabBarCheckbox setState:!PREF_STATE(@"TabHideSingleTab")];
	[_tabKeyEquivalentPopUpMenu selectItemWithTag:[Chax integerForKey:@"TabKeyEquivalent"]];
	[_tabBarLocationPopUpMenu selectItemWithTag:[Chax integerForKey:@"TabBarLocation"]];
	[_tabDisableCloseCheckbox setState:PREF_STATE(@"TabDisableCloseButton")];
	[_tabDisableOverflowMenuCheckbox setState:PREF_STATE(@"TabDisableOverflowMenu")];
	[_tabDisableTearOffCheckbox setState:PREF_STATE(@"TabDisableTearOff")];
	
	//About tab
	[_autoUpdateCheckbox setState:PREF_STATE(@"AutoUpdateChax")];
	
	//Set the minimum value of the formatter to the idle timeout
	NSDictionary *agent = [[NSUserDefaults standardUserDefaults] persistentDomainForName:@"com.apple.ichatagent"];
	NSNumber *idleNumber = [agent objectForKey:@"UserIdleTimeout"];
	double idleTimeout = [idleNumber doubleValue];
	if (idleNumber == nil) {
		idleTimeout = 600.0;
	}
	[[_autoAwayTextField formatter] setMinimum:[NSNumber numberWithDouble:idleTimeout / 60.0]];
}

- (id)init
{
	if ( (self = [super init]) ) {
		_icon = [[NSImage alloc] initWithContentsOfFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"Chax"]];
		[NSBundle loadNibNamed:[self preferencesNibName] owner:self];
		
		NSData *data = [Chax dataForKey:@"ContactListFont"];
		NSFont *font = nil;
		_changingFont = 101;
		if (data != nil) { 
			font = [NSUnarchiver unarchiveObjectWithData:data];
		}
		[self setContactFont:font ? font : [NSFont fontWithName:@"LucidaGrande" size:12]];
		
		data = [Chax dataForKey:@"ContactListStatusFont"];
		font = (data != nil) ? [NSUnarchiver unarchiveObjectWithData:data] : nil;
		_changingFont = 110;
		[self setContactFont:font ? font : [NSFont fontWithName:@"LucidaGrande" size:11]];
		
		data = [Chax dataForKey:@"ContactListGroupFont"];
		font = (data != nil) ? [NSUnarchiver unarchiveObjectWithData:data] : nil;
		_changingFont = 115;
		[self setContactFont:font ? font : [NSFont fontWithName:@"LucidaGrande-Bold" size:13]];
		
		[(Hax_PrefsView *)_view setController:self];
	}
	return self;
}

- (void)dealloc
{
	[_icon release];
	[_contactFont release];
	[_statusMessageFont release];
	[_groupFont release];
	[super dealloc];
}

- (NSView *)viewForPreferenceNamed:(NSString *)name
{
	return _view;
}

- (NSImage *)imageForPreferenceNamed:(NSString *)name
{
	return _icon;
}

- (NSString *)preferencesNibName
{
    return @"Hax_Preferences";
}

- (BOOL)isResizable
{
    return NO;
}

- (NSSize)minSize
{
	return [_view frame].size;
}

- (void)willBeDisplayed
{
	[self performSelector:@selector(sheet) withObject:nil afterDelay:0.0];
}

- (void)sheet
{
	if ([Chax integerForKey:@"Install"] > 0) {
		long difference = [[NSDate date] timeIntervalSinceReferenceDate] - [Chax integerForKey:@"Install"];
		if (difference > 1296000) {
			[Chax setInteger:0 forKey:@"Install"];
			NSBeginInformationalAlertSheet(@"Donate", @"Donate", @"OK", nil, [[FezPreferences sharedPreferences] preferencesPanel], self, @selector(donateSheetDidEnd:returnCode:contextInfo:), nil, nil, @"Please consider donating if you find Chax a useful program. Thank you!");
		}
	}
}

- (void)donateSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton) {
		[[NSWorkspace sharedWorkspace] openURL:DONATE_URL];
	}
}

- (void)registerDefaults
{
	[Chax setBool:NO forKey:@"HideSmileyButton"];
	[Chax setObject:[NSArchiver archivedDataWithRootObject:[NSColor whiteColor]] forKey:@"DockForegroundColor"];
	[Chax setObject:[NSArchiver archivedDataWithRootObject:[NSColor redColor]] forKey:@"DockBackgroundColor"];
}

- (IBAction)changeFileTransferSetting:(id)sender
{
	switch ([sender tag]) {
		case 99:
			if ([_transferGroupMenu indexOfSelectedItem] == 0) {
				[Chax setObject:@"none" forKey:@"AutoAcceptFiles.Group"];
			} else {
				//Set the default to the unique ID of the group
				[Chax setObject:[[[[ABAddressBook sharedAddressBook] groups] objectAtIndex:[_transferGroupMenu indexOfSelectedItem] - 2] uniqueId] forKey:@"AutoAcceptFiles.Group"];
			}
			
			[Chax setBool:([_transferAIMCheckbox state] == NSOnState) forKey:@"AutoAcceptFiles.AIM"];
			[Chax setBool:([_transferJabberCheckbox state] == NSOnState) forKey:@"AutoAcceptFiles.Jabber"];
			[Chax setBool:([_transferBonjourCheckbox state] == NSOnState) forKey:@"AutoAcceptFiles.Bonjour"];
		case 98:
			[NSApp endSheet:_fileTransferOptions];
			[_fileTransferOptions orderOut:nil];
			break;
	}
}

- (IBAction)changeSetting:(id)sender
{
	NSString *key = nil;
	NSMutableDictionary *dict;
	BOOL enable;
	
	switch ([sender tag]) {
		case 101:
			_changingFont = 101;
			[[NSFontManager sharedFontManager] setSelectedFont:_contactFont isMultiple:NO];
			[[NSFontManager sharedFontManager] orderFrontFontPanel:self];
			[[[NSPreferences sharedPreferences] preferencesPanel] makeFirstResponder:_view];
			break;
		case 102:
			[Chax setBool:!SENDER_STATE forKey:@"HideMusicStoreLinks"];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
			break;
		case 103:
			{
				double temp = [_autoAwayTextField doubleValue];
				[AutoAway setAutoAwayEnabled:SENDER_STATE];
				[AutoAway setAutoAwayDelay:temp * 60.0];
				[_autoAwayTextField setEnabled:SENDER_STATE];
				[_autoAwayTextField setDoubleValue:temp];
			}
			break;
		case 104:
			[AutoAway setAutoAwayDelay:[sender doubleValue] * 60.0];
			break;
		case 105:
			enable = SENDER_STATE;
			[Chax setBool:enable forKey:@"UseCustomContactListFonts"];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
			[_setContactFontButton setEnabled:enable];
			[_setStatusMessageFontButton setEnabled:enable];
			[_setGroupFontButton setEnabled:enable];
			break;
		case 106:
			[Chax setBool:SENDER_STATE forKey:@"UseNicknames"];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
			break;
		case 110:
			_changingFont = 110;
			[[NSFontManager sharedFontManager] setSelectedFont:_statusMessageFont isMultiple:NO];
			[[NSFontManager sharedFontManager] orderFrontFontPanel:self];
			[[[NSPreferences sharedPreferences] preferencesPanel] makeFirstResponder:_view];
			break;
		case 111:
			[Chax setBool:SENDER_STATE forKey:@"HideMyAVStatus"];
			if (!_AVRestartWarningShown) {
				NSRunInformationalAlertPanel(LocalizedString(@"my_av_status_toggle_title"), LocalizedString(@"my_av_status_toggle_msg"), nil, nil, nil);
				_AVRestartWarningShown = YES;
			}
			break;
		case 112:
			[Chax setBool:SENDER_STATE forKey:@"AttemptAutoReconnect"];
			break;
		case 113:
			[Chax setBool:SENDER_STATE forKey:@"AnimateIcons"];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
			break;
		case 114:
			[Chax setBool:SENDER_STATE forKey:@"HideOnDeactivate"];
			break;
		case 115:
			_changingFont = 115;
			[[NSFontManager sharedFontManager] setSelectedFont:_groupFont isMultiple:NO];
			[[NSFontManager sharedFontManager] orderFrontFontPanel:self];
			[[[NSPreferences sharedPreferences] preferencesPanel] makeFirstResponder:_view];
			break;
		case 116:
			[Chax setBool:SENDER_STATE forKey:@"ShowMenuBadge"];
			
			//Reload the icon badges
			NSImage *badgeImage = SENDER_STATE ? [[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"menu-badge"]] : nil;
			
			[[[[[NSApp mainMenu] itemAtIndex:3] submenu] itemWithTag:13371] setImage:badgeImage];
			[[[[[NSApp mainMenu] itemAtIndex:4] submenu] itemWithTag:91122] setImage:badgeImage];
			[[[NSApp windowsMenu] itemWithTag:13374] setImage:badgeImage];
			[[[NSApp windowsMenu] itemWithTag:13373] setImage:badgeImage];
			[[[NSApp windowsMenu] itemWithTag:13372] setImage:badgeImage];
			[[[NSApp windowsMenu] itemWithTag:13381] setImage:badgeImage];
			[[[NSApp windowsMenu] itemWithTag:13380] setImage:badgeImage];
			break;
		case 117:
			[Chax setBool:SENDER_STATE forKey:@"AutoresizeContactList"];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
			break;
		case 118:
			[Chax setBool:SENDER_STATE forKey:@"DisableSoundWhenAway"];
			break;
		case 119:
			[Chax setBool:SENDER_STATE forKey:@"DisableSoundWhenAvailable"];
			break;
		case 120:
			[Chax setBool:SENDER_STATE forKey:@"DisableSoundWhenIdle"];
			break;
		case 121:
			[Chax setBool:SENDER_STATE forKey:@"ShowAIMOnReactivate"];
			break;
		case 122:
			[Chax setBool:SENDER_STATE forKey:@"ShowJabberOnReactivate"];
			break;
		case 123:
			[Chax setBool:SENDER_STATE forKey:@"ShowBonjourOnReactivate"];
			break;
		case 301:
			[Chax setBool:SENDER_STATE forKey:@"SkipNewMessageNotification"];
			[Prefs setKnockKnock:!(SENDER_STATE || [Chax boolForKey:@"EnableTabs"])];
			break;
		case 302:
			[Chax setBool:SENDER_STATE forKey:@"ShowStatusChanges"];
			[_statusOptionsButton setEnabled:SENDER_STATE];
			break;
		case 303:
			[Chax setBool:!SENDER_STATE forKey:@"HideSmileyButton"];
			{
				NSEnumerator *docs = [[NSApp orderedDocuments] objectEnumerator];
				ChatDoc *nextDocument;
				
				while ( (nextDocument = [docs nextObject]) ) {
					HaxChatController *controller = [nextDocument controller];
					if ([controller isKindOfClass:[ChatController class]]) {
						[(HaxChatInputLine *)[controller inputLine] setSmileyButtonVisible:(SENDER_STATE ? YES : NO)];
					}
				}
			}
		case 304:
			[Chax setBool:SENDER_STATE forKey:@"WarnMobile"];
			break;
		case 305:
			enable = SENDER_STATE;
			if (enable && (NSRunAlertPanel(LocalizedString(@"auto_accept_files_title"), LocalizedString(@"auto_accept_files_msg"), @"No", @"Yes", nil) == NSAlertDefaultReturn)) {
				enable = NO;
				[(NSCell *)sender setState:NSOffState];
			}
			[_optionsButton setEnabled:enable];
			[Chax setBool:enable forKey:@"AutoAcceptFiles"];
			
			//Register the AutoAccept defaults if they don't exist
			if (![Chax objectForKey:@"AutoAcceptFiles.Group"]) {
				[Chax setObject:@"none" forKey:@"AutoAcceptFiles.Group"];
				[Chax setBool:YES forKey:@"AutoAcceptFiles.AIM"];
				[Chax setBool:YES forKey:@"AutoAcceptFiles.Jabber"];
				[Chax setBool:YES forKey:@"AutoAcceptFiles.Bonjour"];
			}
			break;
		case 306:
			//Load the available AB groups
			[_transferGroupMenu removeAllItems];
			[_transferGroupMenu addItemWithTitle:LocalizedString(@"no_group")];
			NSArray *groups = [[ABAddressBook sharedAddressBook] groups];
			NSEnumerator *enumerator = [groups objectEnumerator];
			ABGroup *nextGroup = nil;
			
			[_transferGroupMenu selectItemAtIndex:0]; //Set the default selected item to the no groups option
			
			if ([groups count] > 0) {
				[[_transferGroupMenu menu] addItem:[NSMenuItem separatorItem]];
				while ( (nextGroup = [enumerator nextObject]) ) {
					[_transferGroupMenu addItemWithTitle:[nextGroup valueForProperty:kABGroupNameProperty]];
					if ([[nextGroup uniqueId] isEqualToString:[Chax stringForKey:@"AutoAcceptFiles.Group"]]) {
						[_transferGroupMenu selectItem:[_transferGroupMenu lastItem]];
					}
				}
			}
			
			//Register the AutoAccept defaults if they don't exist
			if (![Chax objectForKey:@"AutoAcceptFiles.Group"]) {
				[Chax setObject:@"none" forKey:@"AutoAcceptFiles.Group"];
				[Chax setBool:YES forKey:@"AutoAcceptFiles.AIM"];
				[Chax setBool:YES forKey:@"AutoAcceptFiles.Jabber"];
				[Chax setBool:YES forKey:@"AutoAcceptFiles.Bonjour"];
			}
			
			//Set the current defaults for the auto-accept files options
			[_transferAIMCheckbox setState:PREF_STATE(@"AutoAcceptFiles.AIM")];
			[_transferJabberCheckbox setState:PREF_STATE(@"AutoAcceptFiles.Jabber")];
			[_transferBonjourCheckbox setState:PREF_STATE(@"AutoAcceptFiles.Bonjour")];
			[NSApp beginSheet:_fileTransferOptions modalForWindow:[[FezPreferences sharedPreferences] preferencesPanel] modalDelegate:self didEndSelector:NULL contextInfo:NULL];
			break;
		case 307:
			[Chax setBool:SENDER_STATE forKey:@"ConfirmQuit"];
			break;
		case 308:
			[Chax setBool:SENDER_STATE forKey:@"UsePreviewForImages"];
			break;
		case 309:
			[Chax setBool:SENDER_STATE forKey:@"UnblockAVFileTransfers"];
			break;
		case 310:
			if (SENDER_STATE && ![Chax boolForKey:@"SkipNewMessageNotification"]) {
				NSRunInformationalAlertPanel(LocalizedString(@"save_window_frames_title"), LocalizedString(@"save_window_frames_msg"), nil, nil, nil);
			}
			[Chax setBool:SENDER_STATE forKey:@"SaveWindowFrames"];
			break;
		case 311:
			[Chax setBool:SENDER_STATE forKey:@"DisablePIP"];
			break;
		case 312:
			[_statusChangeAvailableCheckbox setState:!PREF_STATE(@"ShowStatusChanges.AvailableOff")];
			[_statusChangeAwayCheckbox setState:!PREF_STATE(@"ShowStatusChanges.AwayOff")];
			[_statusChangeAway2Checkbox setState:!PREF_STATE(@"ShowStatusChanges.AwayChangeOff")];
			[_statusChangeIdleCheckbox setState:!PREF_STATE(@"ShowStatusChanges.IdleOff")];
			[_statusChangeMessageCheckbox setState:!PREF_STATE(@"ShowStatusChanges.MessageChangeOff")];
			[_statusChangeOfflineCheckbox setState:!PREF_STATE(@"ShowStatusChanges.OfflineOff")];
			[_statusChangeOnlineCheckbox setState:!PREF_STATE(@"ShowStatusChanges.OnlineOff")];
			[_statusChangeSongCheckbox setState:!PREF_STATE(@"ShowStatusChanges.SongsOff")];
			
			[NSApp beginSheet:_statusChangeOptions modalForWindow:[[FezPreferences sharedPreferences] preferencesPanel] modalDelegate:self didEndSelector:NULL contextInfo:NULL];
			break;
		case 401:
			enable = SENDER_STATE;
			[_flashBadgeCheckbox setEnabled:enable];
			[_dockNotificationTypeMatrix setEnabled:enable];
			[_dockTextColorWell setEnabled:enable];
			[_dockBackgroundColorWell setEnabled:enable];
			[_previewDockNotificationButton setEnabled:enable];
			[Chax setBool:enable forKey:@"EnableDockNotification"];
			break;
		case 402:
			[Chax setInteger:[sender selectedRow] forKey:@"DockNotificationType"];
			break;
		case 403:
		case 404:
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(colorPanelNotification:) name:NSColorPanelColorDidChangeNotification object:nil];
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(colorPanelNotification:) name:NSWindowWillCloseNotification object:[NSColorPanel sharedColorPanel]];
			break;
		case 405:
			[Chax setBool:SENDER_STATE forKey:@"FlashBadge"];
			break;
		case 406:
			[Chax setBool:SENDER_STATE forKey:@"UseBuddyIconNotification"];
			break;
		case 407:
			[Chax setBool:SENDER_STATE forKey:@"DeminiaturizeOnNewMessages"];
			break;
		case 408:
			if (SENDER_STATE) {
				NSRunInformationalAlertPanel(LocalizedString(@"growl_prefs_title"), LocalizedString(@"growl_prefs_msg"), nil, nil, nil);
			}
			
			[Chax setBool:SENDER_STATE forKey:@"GrowlEnabled"];
			break;
		case 499:
			[[DockIconController sharedController] testDockIcon];
			break;
		case 500:
			[[UpdateController sharedController] checkForUpdates:nil];
			break;
		case 501:
			enable = SENDER_STATE;
			[Chax setBool:enable forKey:@"AutoUpdateChax"];
			[[UpdateController sharedController] setAutomaticCheckingEnabled:enable];
			break;
		case 502:
			[[NSWorkspace sharedWorkspace] openURL:DONATE_URL];
			break;
		case 600:
			key = [_customAutoAwayTextField stringValue];
			enable = SENDER_STATE;
			[Chax setBool:enable forKey:@"CustomAutoAway"];
			[Chax setObject:key forKey:@"CustomAutoAwayMessage"];
			[_customAutoAwayTextField setEnabled:enable];
			[_customAutoAwayTextField setStringValue:key];
			break;
		case 601:
			[Chax setObject:[_customAutoAwayTextField stringValue] forKey:@"CustomAutoAwayMessage"];
			break;
		case 602:
			[Chax setBool:SENDER_STATE forKey:@"AutoReplyOnce"];
			break;
		case 603:
			key = [_awayMessageOnScreenSaverTextField stringValue];
			enable = SENDER_STATE;
			[Chax setBool:SENDER_STATE forKey:@"AwayOnScreenSaver"];
			[Chax setObject:key forKey:@"AwayOnScreenSaverMessage"];
			[_awayMessageOnScreenSaverTextField setEnabled:enable];
			[_unawayOnScreenSaverCheckbox setEnabled:enable];
			[_awayMessageOnScreenSaverTextField setStringValue:key];
			break;
		case 604:
			[Chax setObject:[_awayMessageOnScreenSaverTextField stringValue] forKey:@"AwayOnScreenSaverMessage"];
			break;
		case 605:
			[Chax setBool:SENDER_STATE forKey:@"UnawayOnScreenSaver"];
			break;
	}
}

- (IBAction)changeStatusChangeSetting:(id)sender
{
	switch ([sender tag]) {
		case 99:
			[Chax setBool:([_statusChangeAvailableCheckbox state] == NSOffState) forKey:@"ShowStatusChanges.AvailableOff"];
			[Chax setBool:([_statusChangeAwayCheckbox state] == NSOffState) forKey:@"ShowStatusChanges.AwayOff"];
			[Chax setBool:([_statusChangeAway2Checkbox state] == NSOffState) forKey:@"ShowStatusChanges.AwayChangeOff"];
			[Chax setBool:([_statusChangeIdleCheckbox state] == NSOffState) forKey:@"ShowStatusChanges.IdleOff"];
			[Chax setBool:([_statusChangeMessageCheckbox state] == NSOffState) forKey:@"ShowStatusChanges.MessageChangeOff"];
			[Chax setBool:([_statusChangeOfflineCheckbox state] == NSOffState) forKey:@"ShowStatusChanges.OfflineOff"];
			[Chax setBool:([_statusChangeOnlineCheckbox state] == NSOffState) forKey:@"ShowStatusChanges.OnlineOff"];
			[Chax setBool:([_statusChangeSongCheckbox state] == NSOffState) forKey:@"ShowStatusChanges.SongsOff"];
		case 98:
			[NSApp endSheet:_statusChangeOptions];
			[_statusChangeOptions orderOut:nil];
			break;
	}
}

- (IBAction)changeTabSetting:(id)sender
{
	switch ([sender tag]) {
		case 1:
			[Chax setBool:SENDER_STATE forKey:@"EnableTabs"];
			[_tabBringToFrontCheckbox setEnabled:SENDER_STATE];
			[_tabSelectNewMessagesCheckbox setEnabled:(SENDER_STATE && [_tabBringToFrontCheckbox state] == NSOnState)];
			[Prefs setKnockKnock:!([Chax boolForKey:@"SkipNewMessageNotification"] || SENDER_STATE)];
			break;
		case 2:
		case 3:
		case 4:
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(colorPanelNotification:) name:NSColorPanelColorDidChangeNotification object:nil];
			[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(colorPanelNotification:) name:NSWindowWillCloseNotification object:[NSColorPanel sharedColorPanel]];
			break;
		case 5:
			[Chax setBool:SENDER_STATE forKey:@"TabShowUserIcons"];
			[[NSNotificationCenter defaultCenter] postNotificationName:ChaxTabPreferencesChanged object:nil];
			break;
		case 6:
			[Chax setInteger:[[sender selectedItem] tag] forKey:@"TabKeyEquivalent"];
			
			NSMenu *menu = [NSApp windowsMenu];
			NSMenuItem *previousTab = [menu itemWithTag:13381], *nextTab = [menu itemWithTag:13380];
			const unichar leftArrowChar = NSLeftArrowFunctionKey;
			const unichar rightArrowChar = NSRightArrowFunctionKey;
			const unichar upArrowChar = NSUpArrowFunctionKey;
			const unichar downArrowChar = NSDownArrowFunctionKey;
			NSImage *badgeImage = [[NSImage alloc] initByReferencingFile:[[NSBundle bundleWithIdentifier:@"com.ksuther.chax"] pathForImageResource:@"menu-badge"]];
			
			[menu removeItemAtIndex:4];
			[menu removeItemAtIndex:4];
			
			NSMenuItem *menuItem = [[NSApp windowsMenu] insertItemWithTitle:LocalizedString(@"select_prev_tab") action:@selector(previousTab:) keyEquivalent:[NSString stringWithCharacters:&leftArrowChar length:1] atIndex:4];
			if ([Chax boolForKey:@"ShowMenuBadge"]) {
				[menuItem setImage:badgeImage];
			}
			[menuItem setTag:13381];
			
			NSMenuItem *menuItem2 = [[NSApp windowsMenu] insertItemWithTitle:LocalizedString(@"select_next_tab") action:@selector(nextTab:) keyEquivalent:[NSString stringWithCharacters:&rightArrowChar length:1] atIndex:5];
			if ([Chax boolForKey:@"ShowMenuBadge"]) {
				[menuItem2 setImage:badgeImage];
			}
			[menuItem2 setTag:13380];
			
			switch ([[sender selectedItem] tag]) {
				case 0:
					[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
					[menuItem2 setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
					break;
				case 2:
					[menuItem setKeyEquivalent:@"{"];
					[menuItem2 setKeyEquivalent:@"}"];
					break;
				case 3:
					[menuItem setKeyEquivalent:@"["];
					[menuItem2 setKeyEquivalent:@"]"];
					break;
				case 5:
					[menuItem setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
					[menuItem2 setKeyEquivalentModifierMask:NSCommandKeyMask | NSShiftKeyMask];
				case 4:
					[menuItem setKeyEquivalent:[NSString stringWithCharacters:&upArrowChar length:1]];
					[menuItem2 setKeyEquivalent:[NSString stringWithCharacters:&downArrowChar length:1]];
					break;
			}
			
			[badgeImage release];
			break;
		case 7:
			[Chax setInteger:[[sender selectedItem] tag] forKey:@"TabBarLocation"];
			[[NSNotificationCenter defaultCenter] postNotificationName:ChaxTabPreferencesChanged object:nil];
			break;
		case 8:
			[Chax setBool:!SENDER_STATE forKey:@"TabHideSingleTab"];
			[[NSNotificationCenter defaultCenter] postNotificationName:ChaxTabPreferencesChanged object:nil];
			break;
		case 9:
			[Chax setBool:SENDER_STATE forKey:@"TabFrontOnTextInvitation"];
			[_tabSelectNewMessagesCheckbox setEnabled:SENDER_STATE];
			break;
		case 10:
			[Chax setBool:SENDER_STATE forKey:@"TabSelectNew"];
			[[NSNotificationCenter defaultCenter] postNotificationName:ChaxTabPreferencesChanged object:nil];
			break;
		case 11:
			[Chax setBool:SENDER_STATE forKey:@"TabDisableCloseButton"];
			[[NSNotificationCenter defaultCenter] postNotificationName:ChaxTabPreferencesChanged object:nil];
			break;
		case 12:
			[Chax setBool:SENDER_STATE forKey:@"TabDisableOverflowMenu"];
			[[NSNotificationCenter defaultCenter] postNotificationName:ChaxTabPreferencesChanged object:nil];
			break;
		case 13:
			[Chax setBool:SENDER_STATE forKey:@"TabDisableTearOff"];
			break;
	}
}

- (void)setContactFont:(NSFont *)font
{
	NSTextField *field;
	NSString *key;
	
	switch (_changingFont) {
		case 101:
			[_contactFont release];
			_contactFont = [font retain];
			field = _contactFontTextField;
			key = @"ContactListFont";
			break;
		case 110:
			[_statusMessageFont release];
			_statusMessageFont = [font retain];
			field = _statusMessageFontTextField;
			key = @"ContactListStatusFont";
			break;
		case 115:
			[_groupFont release];
			_groupFont = [font retain];
			field = _groupFontTextField;
			key = @"ContactListGroupFont";
			break;
	}
	
	[field setStringValue:[NSString stringWithFormat:@"%@ %.0f", [font displayName], [font pointSize]]];
	[Chax setObject:[NSArchiver archivedDataWithRootObject:font] forKey:key];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"ReloadContactList" object:nil];
}

- (NSFont *)contactFont
{
	return _contactFont;
}

- (void)colorPanelNotification:(NSNotification *)notification
{
	if (![[notification name] isEqualToString:NSColorPanelColorDidChangeNotification]) {
		[[NSNotificationCenter defaultCenter] removeObserver:self];
	}
	[Chax setObject:[NSArchiver archivedDataWithRootObject:[_dockTextColorWell color]] forKey:@"DockForegroundColor"];
	[Chax setObject:[NSArchiver archivedDataWithRootObject:[_dockBackgroundColorWell color]] forKey:@"DockBackgroundColor"];
	[Chax setObject:[NSArchiver archivedDataWithRootObject:[_tabUnreadMessagesColorWell color]] forKey:@"TabUnreadColor"];
	[Chax setObject:[NSArchiver archivedDataWithRootObject:[_tabUserTypingColorWell color]] forKey:@"TabTypingColor"];
	[Chax setObject:[NSArchiver archivedDataWithRootObject:[_tabUserOfflineColorWell color]] forKey:@"TabOfflineColor"];
}

@end