/*
 * Hax_Presentity.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Hax_Presentity.h"
#import "MethodSwizzle.h"
#import "Chax.h"

@implementation Presentity (Hax_hax_hax)

+ (void)load
{
	MethodSwizzle([self class], @selector(tooltipString), @selector(hax_tooltipString));
	MethodSwizzle([self class], @selector(name), @selector(hax_name));
	MethodSwizzle([self class], @selector(statusURL), @selector(hax_statusURL));
	MethodSwizzle([self class], @selector(attributedStatusMessage), @selector(hax_attributedStatusMessage));
	MethodSwizzle([self class], @selector(bestPresentity), @selector(hax_bestPresentity));
}

- (id)hax_tooltipString
{
	double idleTime = [self idleTime];
	if (idleTime) {
		NSString *statusName = [self nameOfStatus];
		
		NSMutableArray *lines = [[[super tooltipString] componentsSeparatedByString:@"\n"] mutableCopy];
		NSString *fullString;
		[lines insertObject:[self idleString] atIndex:[lines count] - 1];
		//Remove the original Idle line
		if ([self status] == 2) {
			[lines removeObjectAtIndex:2];
		}
		
		fullString = [lines componentsJoinedByString:@"\n"];
		[lines release];
		return fullString;
	} else {
		return [self hax_tooltipString];
	}
}

- (id)hax_name
{
	if ([Chax boolForKey:@"UseNicknames"]) {
		NSString *nick = [[self abPerson] valueForProperty:kABNicknameProperty];
		if (nick) {
			return nick;
		}
	}
	return [self hax_name];
}

- (id)hax_statusURL
{
	//This appears to be the only thing that controls the iTMS store link.
	//If you return nothing, the link to the store won't appear.
	return [Chax boolForKey:@"HideMusicStoreLinks"] ? nil : [self hax_statusURL];
}

- (id)hax_attributedStatusMessage
{
	if (([[[self service] shortName] isEqualToString:@"AIM"] && [Chax boolForKey:@"BuddyList.HideTextStatus"]) ||
		([[[self service] shortName] isEqualToString:@"Jabber"] && [Chax boolForKey:@"JabberViewer.HideTextStatus"]) ||
		([[[self service] shortName] isEqualToString:@"Bonjour"] && [Chax boolForKey:@"SubNetViewer.HideTextStatus"])) {
		return [[[NSAttributedString alloc] initWithString:@""] autorelease];
	} else {
		if ([Chax boolForKey:@"UseCustomContactListFonts"]) {
			NSData *data = [Chax objectForKey:@"ContactListStatusFont"];
			if (data) {
				NSFont *font = [NSUnarchiver unarchiveObjectWithData:data];
				if (font) {
					NSMutableAttributedString *message = [[self hax_attributedStatusMessage] mutableCopy];
					[message addAttribute:NSFontAttributeName value:font range:NSMakeRange(0, [[message string] length])];
					return [message autorelease];
				}
			}
		}
		return [self hax_attributedStatusMessage];
	}
}

- (id)hax_bestPresentity
{
	//Read the CustomPresentities dictionary from the defaults and see if there is a preferred screenname to message by default
	NSDictionary *presentities = [Chax objectForKey:@"CustomPresentities"], *info;
	NSString *key = [[self abPerson] uniqueId];
	Presentity *presentity = nil;
	if (key == nil) {
		key = [self ID];
	}
	
	info = [presentities objectForKey:key];
	
	if (info) {
		NSEnumerator *enumerator = [[self presentities] objectEnumerator];
		Presentity *next;
		while ( (next = [enumerator nextObject]) ) {
			if ([[next ID] isEqualToString:[info objectForKey:[@"PreferredID." stringByAppendingString:[[self service] shortName]]]] && [next status] != 1) {
				presentity = next;
				break;
			}
		}
	}
	
	if (presentity == nil) {
		presentity = [self hax_bestPresentity];
	}
	return presentity;
}

- (void)setBestPresentity:(id)pres
{
	NSMutableDictionary *presentities = [[Chax objectForKey:@"CustomPresentities"] mutableCopy];
	NSString *key = [[pres abPerson] uniqueId];
	NSDictionary *info;
	NSMutableDictionary *mutableInfo;
	
	if (presentities == nil) {
		presentities = [[NSMutableDictionary alloc] init];
	}
	
	if (key == nil) {
		key = [pres ID];
	}
	info = [presentities objectForKey:key];
	
	if (info) {
		mutableInfo = [info mutableCopy];
		[mutableInfo setObject:[pres ID] forKey:[@"PreferredID." stringByAppendingString:[[self service] shortName]]];
		[presentities setObject:mutableInfo forKey:key];
		[mutableInfo release];
	} else {
		[presentities setObject:[NSDictionary dictionaryWithObject:[pres ID] forKey:[@"PreferredID." stringByAppendingString:[[self service] shortName]]] forKey:key];
	}
	
	[Chax setObject:presentities forKey:@"CustomPresentities"];
	[presentities release];
}

- (NSString *)idleString
{
	long idleTime = (long)[self idleTime];
	
	if (idleTime == 0) {
		return nil;
	}
	
	//Build a good idle time format
	int days, hours, minutes, seconds;
	BOOL rawr = NO;
	NSMutableString *idleString = [[NSMutableString alloc] initWithString:LocalizedString(@"idle_for")];
	[idleString appendString:@" "];
	
	if (idleTime > 86400) {
		days = idleTime / 86400;
		idleTime %= 86400;
		[idleString appendFormat:((days > 1) ? @"%i days" : @"%i day"), days];
		rawr = YES;
	}
	
	if (idleTime > 3600) {
		hours = idleTime / 3600;
		idleTime %= 3600;
		if (rawr) {
			[idleString appendString:@", "];
		}
		[idleString appendFormat:((hours > 1) ? @"%i hours" : @"%i hour"), hours];
		rawr = YES;
	}
	
	if (idleTime > 60) {
		minutes = idleTime / 60;
		idleTime %= 60;
		if (rawr) {
			[idleString appendString:@", "];
		}
		[idleString appendFormat:((minutes > 1) ? @"%i minutes" : @"%i minute"), minutes];
		rawr = YES;
	}
	
	if (idleTime) {
		seconds = idleTime;
		if (rawr) {
			[idleString appendString:@", "];
		}
		[idleString appendFormat:((seconds > 1) ? @"%i seconds" : @"%i second"), seconds];
	}
	return [idleString autorelease];
}

@end