/*
 * Hax_Service.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "DockIconController.h"
#import "TabbedChatController.h"
#import "MethodSwizzle.h"
#import "Chax.h"
#import "iChat.h"
#import "HaxChatController.h"
#import <Growl/Growl.h>

@implementation Service (Hax_hax_hax)

+ (void)load
{
	MethodSwizzle([self class], @selector(service:chat:messageReceived:), @selector(hax_service:chat:messageReceived:));
	MethodSwizzle([self class], @selector(service:directIMRequestFrom:invitation:), @selector(hax_service:directIMRequestFrom:invitation:));
	MethodSwizzle([self class], @selector(service:invitedToChat:isChatRoom:invitation:), @selector(hax_service:invitedToChat:isChatRoom:invitation:));
	MethodSwizzle([self class], @selector(_displayDisconnectAlert:wasConnected:), @selector(hax_displayDisconnectAlert:wasConnected:));
	MethodSwizzle([self class], @selector(_nowLoggedIn), @selector(hax_nowLoggedIn));
	//MethodSwizzleClass([self class], @selector(serviceWithInternalName:), @selector(hax_serviceWithInternalName:));
}

/*+ (id)hax_serviceWithInternalName:(id)fp8
{
	if ([fp8 isEqualToString:@"Unified"]) {
		return [UnifiedService sharedInstance];
	}
	return [self hax_serviceWithInternalName:fp8];
}*/

- (oneway void)hax_service:(Service *)fp8 chat:(NSString *)fp12 messageReceived:(FZMessage *)fp16
{
	if ([[fp16 sender] isEqualToString:@"aolsystemmsg"] && [Chax boolForKey:@"BlockAOLSystemMessage"]) {
		return;
	}
	
	[self hax_service:fp8 chat:fp12 messageReceived:fp16];
	
	if ([fp16 sender] != nil) {
		Presentity *presentity = [self existingPresentityWithID:[fp16 sender]];	
		
		if ([fp16 incomingFile] && ShouldAutoAcceptFilesFromSender(presentity)) {
			NSArray *messages = [[[[[self _chatWithID:fp12] chatDoc] controller] transcriptView] messages];
			[[[[[self _chatWithID:fp12] chatDoc] controller] transcriptView] clickedOnLink:[messages objectAtIndex:[messages count] - 1] atIndex:0];
		}
		
		if (![presentity isgMe]) {
			int flags = [fp16 flags];
			Chat *chat = [self _chatWithID:fp12];
			InstantMessage *message = [self _createInstantMessage:fp16];
			
			if ([Chax boolForKey:@"DeminiaturizeOnNewMessages"] && !([Chax boolForKey:@"HideOnDeactivate"] && ![NSApp isActive])) {
				HaxChatController *controller = [[chat chatDoc] controller];
				NSWindow *window = [controller isTab] ? [[TabbedChatController sharedController] windowForChatController:controller] : [controller window];
				if ([window isMiniaturized]) {
					[window deminiaturize:self];
				}
			}
			
			if ([[[chat chatDoc] controller] isTab]) {
				int status = 1;
				if (flags == 8) {
					status = 2;
				} else if (flags == 9) {
					status = [[self _chatWithID:fp12] hasUnreadMessages];
				}
				[[TabbedChatController sharedController] setStatus:status ofChat:chat];
			}
			
			if (![NSApp isActive] && !(((flags == 8) || (flags == 9)) && ([fp16 incomingFile] == nil))) {
				if ([Chax boolForKey:@"GrowlEnabled"]) {
					[GrowlApplicationBridge notifyWithTitle:[NSString stringWithFormat:LocalizedString(@"growl_says"), [presentity name]]
												description:[[message text] string] notificationName:@"New message received"
												iconData:[[[presentity picture] image] TIFFRepresentation]
												priority:0
												isSticky:NO
												clickContext:[NSDictionary dictionaryWithObjectsAndKeys:fp12, @"Chat", [fp8 name], @"Service", nil, nil]];
				}
			
				[[DockIconController sharedController] newMessageFromSender:[presentity name] picture:[[presentity picture] image]];
			}
			
			/*if (1) {
				//Post the distributed notification for the Proxi trigger
				NSDictionary *info = [NSDictionary dictionaryWithObjectsAndKeys:
											@"NewMessage", @"Event",
											[presentity name], @"User",
											[[message text] string], @"Message",
											[[[presentity picture] image] TIFFRepresentation], @"Image",
											nil, nil];
				
				[[NSDistributedNotificationCenter defaultCenter] postNotificationName:@"ChaxProxiNotification" object:nil userInfo:info];
			}*/
		}
	}
}

- (oneway void)hax_service:(id)fp8 directIMRequestFrom:(id)fp12 invitation:(id)fp16
{
	NSEnumerator *docs = [[[TabbedChatController sharedController] chatDocs] objectEnumerator];
	ChatDoc *next;
	BOOL exists = NO;
	while ( (next = [docs nextObject]) ) {
		if ([[[next chat] otherPeople] count] == 1 && [[[next chat] otherPerson] isEqual:[fp16 sender]]) {
			exists = YES;
			break;
		}
	}
	
	if (!exists && [Chax boolForKey:@"EnableTabs"] && [Chax boolForKey:@"SkipNewMessageNotification"]) {
		[[TabbedChatController sharedController] doNotSelectNext];
	}
	
	[self hax_service:fp8 directIMRequestFrom:fp12 invitation:fp16];
}

- (oneway void)hax_service:(Service *)fp8 invitedToChat:(NSString *)fp12 isChatRoom:(BOOL)fp16 invitation:(FZMessage *)fp20
{
	if ([[fp20 sender] isEqualToString:@"aolsystemmsg"] && [Chax boolForKey:@"BlockAOLSystemMessage"]) {
		return;
	}
	
	if ([Chax boolForKey:@"EnableTabs"] && [Chax boolForKey:@"SkipNewMessageNotification"]) {
		[[TabbedChatController sharedController] doNotSelectNext];
	}
	
	[self hax_service:fp8 invitedToChat:fp12 isChatRoom:fp16 invitation:fp20];
	
	InstantMessage *message = [self _createInstantMessage:fp20];
	
	if (![message fromMe]) {
		Presentity *sender = [message sender];
		
		if (![NSApp isActive]) {
			
			if ([Chax boolForKey:@"EnableTabs"]) {
				[self performSelector:@selector(highlightNewTab:) withObject:[self _chatWithID:fp12] afterDelay:0.0];
				
				if ([Chax boolForKey:@"DeminiaturizeOnNewMessages"] && !([Chax boolForKey:@"HideOnDeactivate"] && ![NSApp isActive]) && [[[TabbedChatController sharedController] window] isMiniaturized]) {
					[[[TabbedChatController sharedController] window] deminiaturize:self];
				}
			}
			
			if ([Chax boolForKey:@"GrowlEnabled"]) {
				[GrowlApplicationBridge notifyWithTitle:[NSString stringWithFormat:LocalizedString(@"growl_says"), [sender name]]
											description:[[message text] string]
											notificationName:@"Text invitation received"
											iconData:[[[sender picture] image] TIFFRepresentation]
											priority:0
											isSticky:NO
											clickContext:[NSDictionary dictionaryWithObjectsAndKeys:fp12, @"Chat", [fp8 name], @"Service", nil, nil]];
			}
			
			[[DockIconController sharedController] newMessageFromSender:[sender name] picture:[[sender picture] image]];
		}
		
		/*if (1) {
			//Post the distributed notification for the Proxi trigger
			NSDictionary *info = [NSDictionary dictionaryWithObjectsAndKeys:
										@"TextInvitation", @"Event",
										[sender name], @"User",
										[[message text] string], @"Message",
										[[[sender picture] image] TIFFRepresentation], @"Image",
										nil, nil];
			
			[[NSDistributedNotificationCenter defaultCenter] postNotificationName:@"ChaxProxiNotification" object:nil userInfo:info];
		}*/
	}
}

- (void)hax_displayDisconnectAlert:(id)fp8 wasConnected:(BOOL)fp12
{
	[self hax_displayDisconnectAlert:fp8 wasConnected:fp12];
	
	if (fp12 && [Chax boolForKey:@"AttemptAutoReconnect"]) {
		[self performSelector:@selector(login) withObject:nil afterDelay:5.0];
	}
}

- (void)hax_nowLoggedIn
{
	[self hax_nowLoggedIn];
}

- (void)highlightNewTab:(id)sender
{
	[[TabbedChatController sharedController] setStatus:1 ofChat:sender];
}

@end
