/*
 * InfoWindowController.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "InfoWindowController.h"
#import "FloatingInfoWindow.h"
#import "Hax_Presentity.h"
#import "iChat.h"

static InfoWindowController *_sharedController = nil;

@implementation InfoWindowController

+ (InfoWindowController *)sharedController
{
	if (!_sharedController) {
		_sharedController = [[InfoWindowController alloc] init];
	}
	return _sharedController;
}

- (id)init
{
	if (self = [super init]) {
		//Create a new InfoWindow
		FloatingInfoWindow *window = [[FloatingInfoWindow alloc] initWithContentRect:NSMakeRect(0, 0, 350, 120) styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
		[self setWindow:[window autorelease]];
		[window setDelegate:self];
	}
	return self;
}

- (void)dealloc
{
	[super dealloc];
}

- (void)setPresentity:(Presentity *)presentity rowRect:(NSRect)rect
{
	FloatingInfoWindow *window = (FloatingInfoWindow *)[self window];
	
	[window setImage:[[presentity picture] image]];
	
	if ([presentity status] == 1) {
		[window setString:[NSString stringWithFormat:NSLocalizedString(@"%@ is offline.", nil), [presentity name]]];
	} else {
		NSString *message = [presentity statusMessage];
		
		if (message == nil || [message length] == 0) {
			message = [Person nameOfStatus:[presentity status]];
		}
		
		if ([presentity idleTime] > 0) {
			message = [message stringByAppendingFormat:@"\n\n%@", [presentity idleString]];
		}
		
		[window setString:message];
	}
	
	[window repositionWithRect:rect];
}

- (void)showWindow:(id)sender
{
	if ([[self window] isVisible]) {
		if (_fadeTimer) {
			[_fadeTimer invalidate];
		} else {
			return;
		}
	} else {
		[[self window] setAlphaValue:0.0];
	}
	
	[NSApp activateIgnoringOtherApps:YES];
	[super showWindow:sender];
	_fadeTimer = [NSTimer scheduledTimerWithTimeInterval:(1.0f / 30.0f) target:self selector:@selector(fadeInWindow:) userInfo:nil repeats:YES];
}

- (void)fadeInWindow:(NSTimer *)timer
{
	float alpha = [[self window] alphaValue];
	if (alpha < 0.90) {
		[[self window] setAlphaValue:alpha + 0.10];
	} else {
		[timer invalidate];
		_fadeTimer = nil;
	}
}

- (void)fadeOutWindow:(NSTimer *)timer
{
	float alpha = [[self window] alphaValue];
	if (alpha > 0.0) {
		[[self window] setAlphaValue:alpha - 0.10];
	} else {
		[timer invalidate];
		[[self window] orderOut:nil];
		_fadeTimer = nil;
	}
}

- (void)windowDidResignKey:(NSNotification *)note
{
	_fadeTimer = [NSTimer scheduledTimerWithTimeInterval:(1.0f / 30.0f) target:self selector:@selector(fadeOutWindow:) userInfo:nil repeats:YES];
}

@end
