/*
 * Controller.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "Controller.h"
#import <Security/Security.h>
#import <unistd.h>
#import <sys/socket.h>

@implementation Controller

- (void)awakeFromNib
{
	[_window center];
	[self updateInstallationInfo];
	
	if (_location > 0) {
		[self updateUI];
	}
	
	[_window makeKeyAndOrderFront:nil];
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)application
{
	return YES;
}

- (IBAction)install:(id)sender
{
	long version;
	if ((Gestalt(gestaltSystemVersion, &version) == noErr) && ((version < 4163) || (version > 4176))) {
		NSBeginAlertSheet(NSLocalizedString(@"version_title", nil), nil, nil, nil, _window, self, NULL, NULL, nil, NSLocalizedString(@"version_msg", nil), nil);
		return;
	}
	
	if (_location > 0) {
		[self remove:nil];
	}
	
	NSString *basePath = [[self pathForLocation:[_locationMatrix selectedRow] + 1] stringByDeletingLastPathComponent];
	BOOL success = NO;
	
	//try to install into wherever is specified. if it fails, reinstall with authentication
	
	//Install into ~/Library or /Library
	NSString *sourcePath = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/Resources/Chax"];
	BOOL isDir;
	
	if (![[NSFileManager defaultManager] fileExistsAtPath:basePath isDirectory:&isDir]) {
		if (![[NSFileManager defaultManager] createDirectoryAtPath:basePath attributes:nil]) {
			[self throwError:@"error_msg_2"];
		}
	} else if (!isDir) {
		[self throwError:@"error_msg_3"];
	}
	
	success = [[NSFileManager defaultManager] copyPath:sourcePath toPath:[self pathForLocation:[_locationMatrix selectedRow] + 1] handler:nil];
	
	if (!success) {
		AuthorizationRef authorizationRef;
		AuthorizationItem items[1];
		AuthorizationRights rights;
		char *args[2];
		OSStatus err;
		
		err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
		
		if (err != noErr) {
			[self throwError:@"error_msg_1"];
			return;
		}
		
		items[0].name = kAuthorizationRightExecute;
		items[0].value = NULL;
		items[0].valueLength = 0;
		items[0].flags = 0;
		
		args[0] = ([_locationMatrix selectedRow] == 0) ? "u" : "l";
		args[1] = "w";
		
		rights.count = 1;
		rights.items = items;
		
		err = AuthorizationCopyRights(authorizationRef, &rights, kAuthorizationEmptyEnvironment, kAuthorizationFlagInteractionAllowed | kAuthorizationFlagExtendRights, NULL);
		if (err != noErr) {
			return;
		}
		
		err = AuthorizationExecuteWithPrivileges(authorizationRef, [[[NSBundle mainBundle] pathForResource:@"install_tool" ofType:nil] UTF8String], 0, args, NULL);
		if (err != noErr) {
			return;
		}
		
		err = AuthorizationFree(authorizationRef, kAuthorizationFlagDestroyRights);
		if (err != noErr) {
			return;
		}
		
		success = YES;
	}
	
	/*if (![[NSFileManager defaultManager] isWritableFileAtPath:[basePath stringByDeletingLastPathComponent]]) {
		AuthorizationRef authorizationRef;
		AuthorizationItem items[1];
		AuthorizationRights rights;
		char *args[2];
		OSStatus err;
		
		err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
		
		items[0].name = kAuthorizationRightExecute;
		items[0].value = NULL;
		items[0].valueLength = 0;
		items[0].flags = 0;
		
		args[0] = "l";
		args[1] = "w";
		
		rights.count = 1;
		rights.items = items;
		
		err = AuthorizationCopyRights(authorizationRef, &rights, kAuthorizationEmptyEnvironment, kAuthorizationFlagInteractionAllowed | kAuthorizationFlagExtendRights, NULL);
		err = AuthorizationExecuteWithPrivileges(authorizationRef, [[[NSBundle mainBundle] pathForResource:@"install_tool" ofType:nil] UTF8String], 0, args, NULL);
		err = AuthorizationFree(authorizationRef, kAuthorizationFlagDestroyRights);
		
		success = YES;
	} else {
		//Install into ~/Library or /Library
		NSString *sourcePath = [[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/Resources/Chax"];
		BOOL isDir;
		
		if (![[NSFileManager defaultManager] fileExistsAtPath:basePath isDirectory:&isDir]) {
			if (![[NSFileManager defaultManager] createDirectoryAtPath:basePath attributes:nil]) {
				[self throwError:@"error_msg_2"];
			}
		} else if (!isDir) {
			[self throwError:@"error_msg_3"];
		}
		
		success = [[NSFileManager defaultManager] copyPath:sourcePath toPath:[self pathForLocation:[_locationMatrix selectedRow] + 1] handler:nil];
	}*/
	
	if (success) {
		NSBeginAlertSheet(NSLocalizedString(@"success_title", nil), nil, NSLocalizedString(@"quit", nil), nil, _window, self, NULL, @selector(sheetDidEnd:returnCode:contextInfo:), nil, NSLocalizedString(@"success_msg", nil), nil);
		[self updateInstallationInfo];
		
		NSBundle *bundle = [NSBundle bundleWithPath:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/Resources/Chax/Chax.bundle"]];
		if (bundle == nil) {
			NSLog(@"Fatal error in -updateUI.");
		}
		_version = [[[bundle infoDictionary] objectForKey:@"VersionNumber"] longValue];
		_versionString = [[bundle infoDictionary] objectForKey:@"CFBundleVersion"];
		
		[self updateUI];
	} else {
		[self throwError:@"error_msg_1"];
	}
}

- (IBAction)remove:(id)sender
{
	NSString *location = [self pathForLocation:_location];
	BOOL success;
	if (![[NSFileManager defaultManager] isDeletableFileAtPath:location]) {
		AuthorizationRef authorizationRef;
		AuthorizationItem items[1];
		AuthorizationRights rights;
		char *args[2];
		OSStatus err;
		
		err = AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &authorizationRef);
		
		items[0].name = kAuthorizationRightExecute;
		items[0].value = NULL;
		items[0].valueLength = 0;
		items[0].flags = 0;
		
		args[0] = ((_location == 2) ? "l" : "s");
		args[1] = "d";
		
		rights.count = 1;
		rights.items = items;
		
		err = AuthorizationCopyRights(authorizationRef, &rights, kAuthorizationEmptyEnvironment, kAuthorizationFlagInteractionAllowed | kAuthorizationFlagExtendRights, NULL);
		err = AuthorizationExecuteWithPrivileges(authorizationRef, [[[NSBundle mainBundle] pathForResource:@"install_tool" ofType:nil] UTF8String], 0, args, NULL);
		err = AuthorizationFree(authorizationRef, kAuthorizationFlagDestroyRights);
		
		success = YES;
	} else {
		success = [[NSFileManager defaultManager] removeFileAtPath:location handler:nil];
	}
	
	if (success) {
		if (sender != nil) {
			NSBeginAlertSheet(NSLocalizedString(@"remove_title", nil), nil, nil, nil, _window, self, NULL, NULL, nil, NSLocalizedString(@"remove_msg", nil), nil);
		}
		_location = 0;
		[self updateUI];
	} else {
		NSBeginAlertSheet(NSLocalizedString(@"remove_error_title", nil), nil, nil, nil, _window, self, NULL, NULL, nil, NSLocalizedString(@"remove_error_msg", nil), nil);
	}
}

- (IBAction)showReadMe:(id)sender
{
	[[NSWorkspace sharedWorkspace] openFile:[[NSBundle mainBundle] pathForResource:@"Chax Read Me" ofType:@"rtf"] withApplication:@"TextEdit.app"];
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSAlertAlternateReturn) {
		[NSApp terminate:nil];
	}
}

- (void)updateInstallationInfo
{
	NSBundle *bundle;
	
	_location = 0;
	_version = nil;
	_versionString = nil;
	
	//Check if we're installed
	if ( (bundle = [NSBundle bundleWithPath:[@"~/Library/InputManagers/Chax/Chax.bundle" stringByExpandingTildeInPath]]) ) {
		_location = 1;
	} else if ( (bundle = [NSBundle bundleWithPath:@"/Library/InputManagers/Chax/Chax.bundle"]) ) {
		_location = 2;
	} else if ( (bundle = [NSBundle bundleWithPath:@"/System/Library/InputManagers/Chax/Chax.bundle"]) ) {
		_location = 3;
	}
	
	if (bundle != nil) {
		_version = [[[bundle infoDictionary] objectForKey:@"VersionNumber"] longValue];
		_versionString = [[bundle infoDictionary] objectForKey:@"CFBundleVersion"];
		if (_version == 0) {
			_version = 16809984;
		}
	}
}

- (NSString *)pathForLocation:(int)loc
{
	NSString *location = nil;
	switch (loc) {
		case 1:
			location = [@"~/Library/InputManagers/Chax" stringByExpandingTildeInPath];
			break;
		case 2:
			location = @"/Library/InputManagers/Chax";
			break;
		case 3:
			location = @"/System/Library/InputManagers/Chax";
			break;
	}
	return location;
}

- (void)throwError:(NSString *)message
{
	NSBeginAlertSheet(NSLocalizedString(@"error_title", nil), nil, nil, nil, _window, self, NULL, NULL, nil, NSLocalizedString(message, nil), nil);
}

- (void)updateUI
{
	if (_location > 0) {
		NSString *location = nil;
		switch (_location) {
			case 1:
				location = NSLocalizedString(@"user_library", nil);
				break;
			case 2:
				location = NSLocalizedString(@"main_library", nil);
				break;
			case 3:
				location = NSLocalizedString(@"system_library", nil);
				break;
		}
		
		//Get the version the installer contains
		NSBundle *bundle = [NSBundle bundleWithPath:[[[NSBundle mainBundle] bundlePath] stringByAppendingPathComponent:@"Contents/Resources/Chax/Chax.bundle"]];
		long currentVersion;
		
		if (bundle == nil) {
			NSLog(@"Fatal error in -updateUI.");
		}
		
		currentVersion = [[[bundle infoDictionary] objectForKey:@"VersionNumber"] longValue];
		if (currentVersion > _version) {
			[_installText setStringValue:[NSString stringWithFormat:NSLocalizedString(@"update_msg", nil), location, _versionString, [[bundle infoDictionary] objectForKey:@"CFBundleVersion"]]];
			[_installTitle setStringValue:NSLocalizedString(@"update_title", nil)];
			[_installButton setTitle:NSLocalizedString(@"update", nil)];
		} else {
			[_installText setStringValue:[NSString stringWithFormat:NSLocalizedString(@"installed_msg", nil), location]];
			[_installTitle setStringValue:NSLocalizedString(@"installed_title", nil)];
			[_installButton setTitle:NSLocalizedString(@"install", nil)];
		}
	} else {
		[_installTitle setStringValue:NSLocalizedString(@"install_title", nil)];
		[_installText setStringValue:NSLocalizedString(@"install_msg", nil)];
		[_installButton setTitle:NSLocalizedString(@"install", nil)];
	}
	[_removeButton setEnabled:(_location != 0)];
}

@end