/*
 * LogFile.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import "LogFile.h"

@implementation LogFile

+ (LogFile *)logFileWithName:(NSString *)name fileName:(NSString *)fileName date:(NSDate *)date
{
	return [[[LogFile alloc] initWithName:name fileName:fileName date:date] autorelease];
}

- (id)initWithName:(NSString *)name fileName:(NSString *)fileName date:(NSDate *)date
{
	if ( (self = [super init]) ) {
		_name = [name retain];
		_fileName = [fileName retain];
		_date = [date retain];
	}
	return self;
}

- (void)dealloc
{
	[_name release];
	[_fileName release];
	[_date release];
	[super dealloc];
}

- (NSComparisonResult)compare:(LogFile *)anotherLog
{
	NSComparisonResult result = [[self date] compare:[anotherLog date]];
	
	if (result == NSOrderedAscending) {
		result = NSOrderedDescending;
	} else if (result == NSOrderedDescending) {
		result = NSOrderedAscending;
	}
	
	return result;
}

- (NSString *)name
{
	return _name;
}

- (NSString *)fileName
{
	return _fileName;
}

- (NSDate *)date
{
	return _date;
}

@end
