/*
 * TabbedChatController.h
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#import <Cocoa/Cocoa.h>

#define ChaxTabPreferencesChanged @"ChaxTabPreferencesChanged"

typedef enum {
	BottomPosition = 0,
	TopPosition,
	LeftPosition,
	RightPosition,
} TabPosition;

@class ChatController, Chat, PSMTabBarControl;

@interface TabbedChatController : NSWindowController
{
	IBOutlet NSTabView *_tabView;
	PSMTabBarControl *_tabBarControl;
	NSSplitView *_splitView; //used for vertical tabs
	NSView *_subview;
	BOOL _closeButtonClicked;
	TabPosition _tabPosition;
}
+ (TabbedChatController *)sharedController;

- (void)addChatController:(ChatController *)chatController chatView:(NSView *)chatView drawer:(NSDrawer *)drawer;
- (void)showChatController:(ChatController *)chatController;
- (BOOL)containsChatController:(ChatController *)chatController;
- (BOOL)removeChatController:(ChatController *)chatController;
- (NSWindow *)windowForChatController:(ChatController *)chatController;
- (NSArray *)chatDocs;
- (Chat *)selectedChat;
- (PSMTabBarControl *)tabBar;
- (void)reloadLabels;
- (void)setTabBarPosition:(int)position animate:(BOOL)animate;
- (void)doNotSelectNext;
- (void)doNotSelectNextOff;

- (void)closeTab:(id)sender;

- (void)setStatus:(int)status ofChat:(Chat *)chat;

- (NSMenu *)tabView:(NSTabView *)tabView menuForTabViewItem:(NSTabViewItem *)tabViewItem;

//Forwarded methods
- (void)toggleParticipants:(id)fp8;
- (void)toggleHidePictures:(id)fp8;
- (void)toggleHideAudioStatus:(id)fp8;
- (void)toggleHideVideoStatus:(id)fp8;
- (void)sortByStatus:(id)fp8;
- (void)sortByFirstName:(id)fp8;
- (void)sortByLastName:(id)fp8;
- (void)addLink:(id)fp8;
- (void)composeEMail:(id)fp8;
- (void)sendSong:(id)fp8;
- (void)sendFile:(id)fp8;
- (void)insertSmiley:(id)fp8;
- (void)setRendererFromMenuItem:(id)fp8;
- (void)inviteToVideoChat:(id)fp8;
- (void)inviteToAudioChat:(id)fp8;
- (void)toggleHideSmileys:(id)fp8;
- (void)chooseBackgroundImage:(id)fp8;
- (void)clearBackground:(id)fp8;
- (void)toggleShowNames:(id)fp8;
- (void)toggleShowPictures:(id)fp8;
- (void)setChatShowsNames:(id)fp8;
- (void)setChatShowsPictures:(id)fp8;
- (void)setChatShowsNamesAndPictures:(id)fp8;
- (void)chooseChatOptions:(id)fp8;
- (void)sendMessage:(id)fp8;
- (void)sendPartialMessage:(id)fp8;
- (void)cancelMessage:(id)fp8;
- (void)finishChat:(id)fp8;
- (void)ignore:(id)fp8;
- (void)block:(id)fp8;
@end
