/*
 * TabbedChatController.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

/*

Portions of this software Copyright 2004 The Shiira Project. All rights reserved.

Portions of this software Copyright (c) 2005, Positive Spin Media. All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
		Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
		Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
		Neither the name of Positive Spin Media nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
        
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#import <objc/objc-class.h>
#import "TabbedChatController.h"
#import "Chax.h"
#import "iChat.h"
#import "NSImage_SmoothAdditions.h"
#import "HaxChatController.h"
#import "HaxTabViewItem.h"
#import "PSMTabBarControl.h"
#import "PSMOverflowPopUpButton.h"
#import "PSMMetalTabStyle.h"
#import "PSMAquaTabStyle.h"

static TabbedChatController *_sharedController = nil, *_oldController = nil;
static NSMutableArray *_subcontrollers = nil;
static BOOL _doNotSelectNext;

/*@interface TabbedChatController (Private)
- (void)moveTabToWindow:(id)sender;
@end*/

@implementation TabbedChatController

+ (TabbedChatController *)sharedController
{
	if (!_sharedController) {
		_sharedController = [[TabbedChatController alloc] initWithWindowNibName:@"TabWindow"];
		_subcontrollers = [[NSMutableArray alloc] init];
	}
	return _sharedController;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}

- (void)awakeFromNib
{
	[[self window] setExcludedFromWindowsMenu:YES];
	[_tabView removeTabViewItem:[_tabView selectedTabViewItem]];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSApplicationDidBecomeActiveNotification object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSWindowDidResignKeyNotification object:[self window]];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSWindowDidBecomeKeyNotification object:[self window]];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSApplicationWillTerminateNotification object:nil];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:ChaxTabPreferencesChanged object:nil];
	
	//Set the window position
	if ([Chax stringForKey:@"TabWindowFrame"]) {
		[[self window] setFrameFromString:[Chax stringForKey:@"TabWindowFrame"]];
	}
	
	_closeButtonClicked = NO;
	
	//Add the tab bar control
	NSRect bounds = [[[self window] contentView] bounds];
	_tabBarControl = [[PSMTabBarControl alloc] initWithFrame:NSMakeRect(0, 0, bounds.size.width, 22)];
	[_tabBarControl awakeFromNib];
	[[[self window] contentView] addSubview:[_tabBarControl autorelease]];
	[_tabBarControl setPartnerView:_tabView];
	[_tabBarControl setTabView:_tabView];
	[_tabBarControl setDelegate:self];
	[_tabView setDelegate:_tabBarControl];
	[_tabBarControl setCanCloseOnlyTab:YES];
	[_tabBarControl setStyleNamed:@"Metal"];
	//[_tabBarControl setStyleNamed:@"Aqua"];
	[_tabBarControl setAutoresizingMask:NSViewWidthSizable | ([Chax integerForKey:@"TabBarLocation"] ? NSViewMinYMargin : NSViewMaxYMargin)];
	[_tabBarControl setHideForSingleTab:[Chax boolForKey:@"TabHideSingleTab"]];
	[_tabBarControl setDisableTabClose:[Chax boolForKey:@"TabDisableCloseButton"]];
	[_tabBarControl setUseOverflowMenu:![Chax boolForKey:@"TabDisableOverflowMenu"]];
	[_tabBarControl setSelectsTabsOnMouseDown:YES];
	[_tabBarControl setAutomaticallyAnimates:NO];
	[_tabBarControl setSizeCellsToFit:YES];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(notificationReceived:) name:NSViewFrameDidChangeNotification object:_tabBarControl];
	
	//overflow button alternate image
	NSImage *images = [[NSImage alloc] initByReferencingFile:@"/System/Library/CoreServices/Menu Extras/iChat.menu/Contents/Resources/iChatMenuExtraImages.tiff"];
	
	//Set the new message image on the overflow button
	NSImage *image = [[[NSImage alloc] initWithSize:NSMakeSize(22, 22)] autorelease];
	[image lockFocus];
	[images compositeToPoint:NSMakePoint(0, 0) fromRect:NSMakeRect(44, 22, 22, 22) operation:NSCompositeSourceOver];
	[image unlockFocus];
	
	[images release];
	
	[[_tabBarControl overflowPopUpButton] setAlternateImage:image];
	
	//Tab sizing
	[_tabBarControl setCellMinWidth:70];
	[self setTabBarPosition:[Chax integerForKey:@"TabBarLocation"] animate:NO];
	
	/*[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(tabNotificationReceived:) name:PSMTabDragDidBeginNotification object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(tabNotificationReceived:) name:PSMTabDragDidEndNotification object:nil];*/
}

//
//
//
//
//

- (void)addChatController:(ChatController *)chatController chatView:(NSView *)chatView drawer:(NSDrawer *)drawer
{
	NSWindow *window = [self window];
	
	[self showWindow:self];
	[_tabBarControl setDelegate:self];
	
	if ([_tabView indexOfTabViewItemWithIdentifier:chatController] == NSNotFound) {
		[(HaxChatController *)chatController setIsTab:YES];
		
		//Make a new tab item
		HaxTabViewItem *newTabViewItem = [[[HaxTabViewItem alloc] initWithIdentifier:chatController] autorelease];
		Chat *chat = [chatController chat];
		
		[drawer setParentWindow:window];
		
		//Move the chat view to the tab item
		[chatView retain];
		[chatView removeFromSuperview];
		[newTabViewItem setView:chatView];
		[[chatController transcriptView] restoreDefaultBackground];
		
		//[[chatController inputLine] setTarget:self];
		[[chatController inputLine] setTarget:nil];
		
		//Add and select the new tab
		[_tabView addTabViewItem:newTabViewItem];
		
		if (!_doNotSelectNext) {
			[_tabView selectTabViewItem:newTabViewItem];
		}
		
		if ([[chat otherPerson] status] == 1) {
			[[TabbedChatController sharedController] setStatus:3 ofChat:chat];
		}
		
		if ([_tabView selectedTabViewItem] != newTabViewItem && [chat hasUnreadMessages]) {
			[newTabViewItem setChatStatus:1];
			[_tabBarControl setNeedsDisplay:YES];
		}
	} else {
		[_tabView selectTabViewItemWithIdentifier:chatController];
	}
}

- (void)showChatController:(ChatController *)chatController
{
	if (self == _sharedController) {
		//check all subcontrollers for the tab and show it
		NSEnumerator *enumerator = [_subcontrollers objectEnumerator];
		TabbedChatController *next;
		
		while ( (next = [enumerator nextObject]) ) {
			if ([next containsChatController:chatController]) {
				[next showChatController:chatController];
				return;
			}
		}
	}
	
	if (!_doNotSelectNext) {
		[_tabView selectTabViewItemWithIdentifier:chatController];
		[self showWindow:self];
	} else {
		[self performSelector:@selector(doNotSelectNextOff) withObject:nil afterDelay:0.0];
	}
}

- (BOOL)containsChatController:(ChatController *)chatController
{
	if (self == _sharedController) {
		//check all subcontrollers for the tab
		NSEnumerator *enumerator = [_subcontrollers objectEnumerator];
		TabbedChatController *next;
		
		while ( (next = [enumerator nextObject]) ) {
			if ([next containsChatController:chatController]) {
				return YES;
			}
		}
	}
	
	return [_tabView numberOfTabViewItems] > 0 && [_tabView indexOfTabViewItemWithIdentifier:chatController] != NSNotFound;
}

- (BOOL)removeChatController:(ChatController *)chatController
{
	if (self == _sharedController) {
		//check all subcontrollers for the tab to remove and try to remove it
		NSEnumerator *enumerator = [_subcontrollers objectEnumerator];
		TabbedChatController *next;
		
		while ( (next = [enumerator nextObject]) ) {
			BOOL value = [next removeChatController:chatController];
			
			if (value) {
				return YES;
			}
		}
	}
	
	NSEnumerator *enumerator = [[_tabView tabViewItems] objectEnumerator];
	NSTabViewItem *nextTab;
	while ( (nextTab = [enumerator nextObject]) ) {
		if ([[nextTab identifier] isEqual:chatController]) {
			NSMenuItem *item = [[NSMenuItem alloc] init];
			[item setRepresentedObject:nextTab];
			[self closeTab:item];
			[item release];
			return YES;
		}
	}
	
	return NO;
}

- (NSWindow *)windowForChatController:(ChatController *)chatController
{
	NSWindow *window = nil;
	NSEnumerator *enumerator = [_subcontrollers objectEnumerator];
	TabbedChatController *next;
	
	if ([_tabView numberOfTabViewItems] > 0 && [_tabView indexOfTabViewItemWithIdentifier:chatController] != NSNotFound) {
		window = [self window];
	} else {
		while ( (next = [enumerator nextObject]) ) {
			if ([next containsChatController:chatController]) {
				window = [next window];
				break;
			}
		}
	}
	return window;
}

- (Chat *)selectedChat
{
	return [[[_tabView selectedTabViewItem] identifier] chat];
}

- (PSMTabBarControl *)tabBar
{
	return _tabBarControl;
}

- (NSArray *)chatDocs
{
	NSMutableArray *array = [[NSMutableArray alloc] init];
	NSEnumerator *enumerator = [[_tabView tabViewItems] objectEnumerator];
	NSTabViewItem *nextTab;
	
	if (self == _sharedController) {
		//Add all subcontroller ChatDoc instances if we're the main controller
		NSEnumerator *enumerator = [_subcontrollers objectEnumerator];
		TabbedChatController *next;
		
		while ( (next = [enumerator nextObject]) ) {
			[array addObjectsFromArray:[next chatDocs]];
		}
	}
	
	while ( (nextTab = [enumerator nextObject]) ) {
		[array addObject:[[[nextTab identifier] chat] chatDoc]];
	}
	
	return [array autorelease];
}

- (void)reloadLabels
{
	NSTabViewItem *item = [_tabView selectedTabViewItem];
	[item setLabel:[item label]];
	[[self window] setTitle:[NSString stringWithFormat:LocalizedString(@"tab_window_title"), [item label]]];
}

- (void)doNotSelectNext
{
	_doNotSelectNext = YES;
}

- (void)doNotSelectNextOff
{
	_doNotSelectNext = NO;
}

- (void)setTabBarPosition:(int)position animate:(BOOL)animate
{
	NSRect contentViewFrame = [[[self window] contentView] bounds], tabViewFrame = [_tabView frame], tabBarFrame = [_tabBarControl bounds];
	
	[(PSMMetalTabStyle *)[_tabBarControl style] setDrawsRight:NO];
	
	if ([_tabBarControl orientation] == PSMTabBarVerticalOrientation) {
		[_tabView removeFromSuperview];
		[_tabBarControl removeFromSuperview];
		
		[[[self window] contentView] addSubview:_tabView];
		[[[self window] contentView] addSubview:_tabBarControl];
		
		[_splitView removeFromSuperview];
	}
	
	if (position == BottomPosition) {
		tabBarFrame.origin.x = 0;
		tabBarFrame.origin.y = 0;
		tabBarFrame.size.width = contentViewFrame.size.width;
		tabBarFrame.size.height = 22;
		
		tabViewFrame.origin.x = 4;
		tabViewFrame.origin.y = tabBarFrame.size.height + 7;
		tabViewFrame.size.width = contentViewFrame.size.width - 8;
		tabViewFrame.size.height = contentViewFrame.size.height - tabBarFrame.size.height - 6;
		
		[_tabBarControl setAutoresizingMask:NSViewWidthSizable | NSViewMaxYMargin];
	} else if (position == TopPosition) {
		tabBarFrame.size.width = contentViewFrame.size.width;
		tabBarFrame.size.height = 22;
		tabBarFrame.origin.x = 0;
		tabBarFrame.origin.y = contentViewFrame.size.height - tabBarFrame.size.height;
		
		tabViewFrame.origin.x = 4;
		tabViewFrame.origin.y = 8;
		tabViewFrame.size.width = contentViewFrame.size.width - 8;
		tabViewFrame.size.height = contentViewFrame.size.height - tabBarFrame.size.height - 6;
		
		[_tabBarControl setAutoresizingMask:NSViewWidthSizable | NSViewMinYMargin];
	} else {
		_splitView = [[NSSplitView alloc] initWithFrame:contentViewFrame];
		[_splitView setDelegate:self];
		[_splitView setVertical:YES];
		[_splitView setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
		
		tabViewFrame.origin.x = (position == LeftPosition) ? 6 : 4;
		tabViewFrame.origin.y = 0;
		_subview = [[[NSView alloc] initWithFrame:tabViewFrame] autorelease];
		[_subview setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
		[_subview addSubview:_tabView];
		
		if (position == LeftPosition) {
			[_splitView addSubview:_tabBarControl];
			[_splitView addSubview:_subview];
		} else {
			[_splitView addSubview:_subview];
			[_splitView addSubview:_tabBarControl];
		}
		
		[[[self window] contentView] addSubview:_splitView];
		
		tabBarFrame.origin.y = 0;
		tabBarFrame.origin.x = 0;
		
		float tabWidth = [Chax integerForKey:@"VerticalTabWidth"];
		
		if (tabWidth > 100) {
			tabWidth = 100;
		}
		
		tabBarFrame.size.width = (tabWidth == 0) ? 100 : tabWidth;
		
		tabViewFrame.origin.x = 0;
		tabViewFrame.origin.y = 8;
		tabViewFrame.size.height = contentViewFrame.size.height - 36;
		[_tabView setFrame:tabViewFrame];
		
		if (position == 2) {
			[_tabBarControl setAutoresizingMask:NSViewHeightSizable];
		} else {
			[_tabBarControl setAutoresizingMask:NSViewHeightSizable | NSViewMinXMargin];
			
			[(PSMMetalTabStyle *)[_tabBarControl style] setDrawsRight:YES];
		}
	}
	
	_tabPosition = position;
	
	if (animate && position < 2 && [_tabBarControl orientation] == PSMTabBarHorizontalOrientation) {
		NSDictionary *barDict = [NSDictionary dictionaryWithObjectsAndKeys:_tabBarControl, NSViewAnimationTargetKey,
																			[NSValue valueWithRect:tabBarFrame], NSViewAnimationEndFrameKey, nil],
					*viewDict = [NSDictionary dictionaryWithObjectsAndKeys:_tabView, NSViewAnimationTargetKey, [NSValue valueWithRect:tabViewFrame],
																			NSViewAnimationEndFrameKey, nil];
		NSViewAnimation *animation = [[NSViewAnimation alloc] initWithViewAnimations:[NSArray arrayWithObjects:barDict, viewDict, nil]];
		[animation setAnimationBlockingMode:NSAnimationNonblocking];
		[animation startAnimation];
	} else {
		[_tabView setFrame:tabViewFrame];
		[_tabBarControl setFrame:tabBarFrame];
		[[self window] display];
	}
	
	[_tabBarControl setOrientation:(position < 2) ? PSMTabBarHorizontalOrientation : PSMTabBarVerticalOrientation];

}

- (void)setStatus:(int)status ofChat:(Chat *)chat
{
	//Loop through the tabs to find the tab associated with the presentity
	NSEnumerator *tabEnumerator = [[_tabView tabViewItems] objectEnumerator];
	HaxTabViewItem *nextTab;
	
	//Highlight the given status. If the status is "new message" (1) and the given presentity is the current tab, set the status to 0.
	while ( (nextTab = [tabEnumerator nextObject]) ) {
		if ([[[nextTab identifier] chat] isEqual:chat]) {
			if (status == 3 && [nextTab chatStatus] == 1) {
				//Skip highlighting the offline state if there are new messages waiting
				return;
			}
			
			if (([_tabView selectedTabViewItem] != nextTab || ![NSApp isActive]) && ([[_tabBarControl representedTabViewItems] indexOfObject:nextTab] >= [_tabBarControl numberOfVisibleTabs])) {
				[[_tabBarControl overflowPopUpButton] setAnimatingAlternateImage:[[[nextTab identifier] chat] hasUnreadMessages]];
			}
			
			[nextTab setChatStatus:([[[[_tabView selectedTabViewItem] identifier] chat] isEqual:chat] && (status == 1) && [NSApp isActive]) ? 0 : status];
			[_tabBarControl setNeedsDisplay:YES];
			break;
		}
	}
}

- (int)statusOfChat:(Chat *)chat
{
	//Loop through the tabs to find the tab associated with the presentity
	NSEnumerator *tabEnumerator = [[_tabView tabViewItems] objectEnumerator];
	HaxTabViewItem *nextTab;
	int status = -1;
	
	while ( (nextTab = [tabEnumerator nextObject]) ) {
		if ([[[nextTab identifier] chat] isEqual:chat]) {
			status = [nextTab chatStatus];
		}
	}
	return status;
}


//
//
// Actions
//
//

- (void)closeTab:(id)sender
{
	if ([[sender representedObject] isKindOfClass:[NSTabViewItem class]]) {
		[_tabView removeTabViewItem:[sender representedObject]];
	}
}

/*- (void)moveWindowToTab:(id)sender
{
	HaxChatController *controller = [sender representedObject];
	
	if ([controller isTab]) {
		[sender setRepresentedObject:[_tabView tabViewItemAtIndex:[_tabView indexOfTabViewItemWithIdentifier:controller]]];
		[self moveTabToWindow:sender];
	} else {
		int state = [[controller peopleDrawer] state];
		
		if (![[controller chat] fromMe] && [[controller chat] style] == 2) {
			//The chat isn't accepted yet and is still in the notifier window
			[controller acceptNotification:nil];
		}
		
		[self addChatController:controller chatView:[controller enclosingView] drawer:[controller peopleDrawer]];
		[[controller peopleDrawer] close];
		[[controller window] orderOut:nil];
		
		if (state == NSDrawerOpenState || state == NSDrawerClosingState) {
			[controller performSelector:@selector(showParticipants:) withObject:nil afterDelay:0.5];
		}
	}
}

- (void)moveTabToWindow:(id)sender
{
	NSTabViewItem *item = [sender representedObject];
	NSWindow *window = [[item identifier] window];
	NSView *view = [item view], *subview0 = [[[window contentView] subviews] objectAtIndex:0];
	NSRect windowFrame = [window frame];
	HaxChatController *identifier = [item identifier];
	
	//Remove the tab view item, set the controller to not being tabbed, relocate the view, show the original window
	
	[identifier setIsTab:NO];
	[_tabView removeTabViewItem:item];
	[view retain];
	[view removeFromSuperview];
	[subview0 addSubview:view];
	[view setFrame:NSMakeRect(0, 0, windowFrame.size.width - 8, windowFrame.size.height - 29)];
	
	//Move the drawer
	NSDrawer *drawer = [identifier peopleDrawer];
	int state = [drawer state];
	
	//Reset the input line target
	//[[identifier inputLine] setTarget:identifier];
	[[identifier inputLine] setTarget:nil];
	//[[identifier inputLine] retain];
	
	[drawer close];
	
	//Call this delayed, otherwise we'll crash if you close the window too quickly
	[self performSelector:@selector(postMoveToWindow:) withObject:[NSArray arrayWithObjects:window, drawer, [NSNumber numberWithInt:state], nil] afterDelay:0.5];
	
	[window makeKeyAndOrderFront:nil];
	
	if ([identifier isInputLineVisible]) {
		[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] _setFirstResponder];
	}
	
	if ([_tabView numberOfTabViewItems] == 0) {
		[[self window] close];
	}
}

- (void)postMoveToWindow:(NSArray *)sender
{
	if ([[sender objectAtIndex:0] isVisible]) {
		[[sender objectAtIndex:1] setParentWindow:[sender objectAtIndex:0]];
		if ([[sender objectAtIndex:2] intValue] == NSDrawerOpenState || [[sender objectAtIndex:2] intValue] == NSDrawerClosingState) {
			[[sender objectAtIndex:1] open];
		}
	} else {
		[[sender objectAtIndex:1] setParentWindow:nil];
	}
}*/

- (void)nextTab:(id)sender
{
	NSTabViewItem *selectedItem = [_tabView selectedTabViewItem];
	NSArray *cells = [_tabBarControl cells];
	int i;
	
	for (i = 0; i < [cells count]; i++) {
		if ([[[cells objectAtIndex:i] representedObject] isEqual:selectedItem]) {
			[_tabView selectTabViewItem:[[cells objectAtIndex:((i == [cells count] - 1) ? 0 : i + 1)] representedObject]];
		}
	}
}

- (void)previousTab:(id)sender
{
	NSTabViewItem *selectedItem = [_tabView selectedTabViewItem];
	NSArray *cells = [_tabBarControl cells];
	int i;
	
	for (i = 0; i < [cells count]; i++) {
		if ([[[cells objectAtIndex:i] representedObject] isEqual:selectedItem]) {
			[_tabView selectTabViewItem:[[cells objectAtIndex:((i == 0) ? [cells count] - 1 : i - 1)] representedObject]];
		}
	}
}

- (void)showWindow:(id)sender
{
	if ([NSApp isActive]) {
		[super showWindow:sender];
		return;
	} else if ([_tabView numberOfTabViewItems] == 0) {
		if ([Chax boolForKey:@"HideOnDeactivate"]) {
			[[self window] setHidesOnDeactivate:YES];
		}
		
		[[self window] orderFront:nil];
		[[self window] makeKeyWindow];
		
		if ([Chax boolForKey:@"HideOnDeactivate"]) {
			[[self window] setHidesOnDeactivate:NO];
		}
	}
	
	if ([Chax boolForKey:@"TabFrontOnTextInvitation"] && ![Chax boolForKey:@"HideOnDeactivate"]) {
		[[self window] orderFront:nil];
		[[self window] makeKeyWindow];
		
		if (![NSApp isActive]) {
			_doNotSelectNext = ![Chax boolForKey:@"TabSelectNew"];
			[[self window] setLevel:NSStatusWindowLevel];
			[[self window] setLevel:NSNormalWindowLevel];
		}
	}
}

//
//
//
//
//

- (void)notificationReceived:(NSNotification *)note
{
	if ([[note name] isEqualToString:ChaxTabPreferencesChanged]) {
		[[self tabBar] setHideForSingleTab:[Chax boolForKey:@"TabHideSingleTab"]];
		[[self tabBar] setDisableTabClose:[Chax boolForKey:@"TabDisableCloseButton"]];
		[[self tabBar] setUseOverflowMenu:![Chax boolForKey:@"TabDisableOverflowMenu"]];
		
		if (_tabPosition != [Chax integerForKey:@"TabBarLocation"]) {
			[self setTabBarPosition:[Chax integerForKey:@"TabBarLocation"] animate:YES];
		}
	} else if ([[note name] isEqualToString:NSApplicationDidBecomeActiveNotification]) {
		HaxTabViewItem *item = (HaxTabViewItem *)[_tabView selectedTabViewItem];
		
		if ([item chatStatus] == 1) {
			[item setChatStatus:0];
			
			//See if we need to disable the overflow button animation
			NSArray *tabs = [_tabBarControl representedTabViewItems];
			int i;
			BOOL animate = NO;
			
			for (i = [_tabBarControl numberOfVisibleTabs]; i < [tabs count]; i++) {
				HaxTabViewItem *tab = [tabs objectAtIndex:i];
				
				if ([tab chatStatus] == 1) {
					animate = YES;
					break;
				}
			}
			
			if (!animate) {
				[[_tabBarControl overflowPopUpButton] setAnimatingAlternateImage:NO];
			}
			
			[_tabBarControl setNeedsDisplay];
		}
	} else if ([[note name] isEqualToString:NSApplicationWillTerminateNotification] && self == _sharedController) {
		[Chax setObject:[[self window] stringWithSavedFrame] forKey:@"TabWindowFrame"];
	} else if ([[note name] isEqualToString:NSViewFrameDidChangeNotification] && _tabPosition == LeftPosition || _tabPosition == RightPosition) {
		[Chax setInteger:[_tabBarControl frame].size.width forKey:@"VerticalTabWidth"];
		return;
	}
	[[[[_tabView selectedTabViewItem] identifier] inputLine] setFocusRingNeedsDisplay];
}

- (void)toggleIsOnTop:(id)sender
{
	[[self window] setLevel:([[self window] level] == NSNormalWindowLevel) ? NSStatusWindowLevel : NSNormalWindowLevel];
}

- (BOOL)validateMenuItem:(id <NSMenuItem>)sender
{
	int tag = [sender tag];
	
	/*if (tag == 13390) {
		[sender setTitle:[[sender representedObject] isTab] ? LocalizedString(@"move_to_window") : LocalizedString(@"move_to_tab")];
	} else */
	if (tag == 13374) {
		[sender setState:([[self window] level] == NSNormalWindowLevel) ? NSOffState : NSOnState];
	} else if (tag == 13380 || tag == 13381) {
		return [[self window] isVisible] && ([_tabView numberOfTabViewItems] > 0);
	} else if ([sender action] == @selector(showSelectedBuddyInfo:)) {
		return [[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] presentityToGetInfoOn] != nil;
	} else if ([sender action] == @selector(addABuddy:)) {
		Presentity *presentity = [[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] presentityToGetInfoOn];
		[People retitleAddBuddyItem:sender forPresentity:presentity];
		return presentity && ![presentity isBuddy];
	} else if ([[[_tabView selectedTabViewItem] identifier] respondsToSelector:[sender action]]) {
        return ([[[_tabView selectedTabViewItem] identifier] validateMenuItem:sender]);
    }
	
	return YES;
}

- (void)performClose:(id)sender
{
	//Close the selected tab
	NSTabViewItem *item = [_tabView selectedTabViewItem];
	
	if (item != nil) {
		[_tabView removeTabViewItem:item];
	}
}

- (void)closeWarningSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if (returnCode == NSOKButton) {
		[self performSelector:@selector(closeWindowDelayed) withObject:nil afterDelay:0.0];
	}
}

- (void)closeWindowDelayed
{
	//Remove all tabs
	NSEnumerator *enumerator = [[_tabView tabViewItems] objectEnumerator];
	NSTabViewItem *item;
	while ( (item = [enumerator nextObject]) ) {
		[_tabView removeTabViewItem:item];
	}
	[[self window] close];
}

#pragma mark -
#pragma mark Drag And Drop Methods
#pragma mark -

/*- (void)tabNotificationReceived:(NSNotification *)notification
{
	if ([[notification name] isEqualToString:PSMTabDragDidBeginNotification]) {
		[_tabBarControl setHideForSingleTab:NO];
	} else {
		[[NSNotificationCenter defaultCenter] postNotificationName:ChaxTabPreferencesChanged object:nil];
	}
}*/

#pragma mark -
#pragma mark Split View Delegate
#pragma mark -

- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset
{
	return _tabPosition == LeftPosition ? 40 : [sender frame].size.width - 150 - [sender dividerThickness];
}

- (float)splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset
{
	return _tabPosition == LeftPosition ? 150 : proposedMax - 40;
}

- (void)splitView:(NSSplitView *)sender resizeSubviewsWithOldSize:(NSSize)oldSize
{
	NSRect senderFrame = [sender frame];
	NSRect subviewFrame = [_subview frame], tabBarControlFrame = [_tabBarControl frame];
	
	subviewFrame.size = NSMakeSize(senderFrame.size.width - tabBarControlFrame.size.width - [sender dividerThickness] - 4, [[[self window] contentView] frame].size.height);
	
	tabBarControlFrame.size.height = senderFrame.size.height;
	
	if (_tabPosition == LeftPosition) {
		subviewFrame.origin.x = tabBarControlFrame.size.width + [sender dividerThickness];
	} else {
		tabBarControlFrame.origin.x = senderFrame.size.width - tabBarControlFrame.size.width;
	}
	
	[_subview setFrame:subviewFrame];
	[_tabBarControl setFrame:tabBarControlFrame];
}

#pragma mark -
#pragma mark NSWindow Delegate Methods
#pragma mark -

- (BOOL)windowShouldClose:(id)sender
{
	if ([_tabView numberOfTabViewItems] > 1) {
		//Put up a warning sheet before the window actually closes
		NSBeginAlertSheet(LocalizedString(@"confirm_window_close"), LocalizedString(@"close_all_tabs"), LocalizedString(@"dont_close"), nil, [self window], self, @selector(closeWarningSheetDidEnd:returnCode:contextInfo:), nil , nil, LocalizedString(@"confirm_window_close_msg"));
		return NO;
	}
	
	_closeButtonClicked = YES;
	
	return YES;
}

- (void)windowWillClose:(id)sender
{
	//Remove all tabs
	NSEnumerator *enumerator = [[_tabView tabViewItems] objectEnumerator];
	NSTabViewItem *item;
	
	while ( (item = [enumerator nextObject]) ) {
		[_tabView removeTabViewItem:item];
	}
	
	if (self != _sharedController) {
		[_subcontrollers removeObject:self];
		[self release];
	} else if ([_subcontrollers count]) {
		[_oldController release];
		_oldController = _sharedController;
		_sharedController = [_subcontrollers objectAtIndex:0];
		[_subcontrollers removeObjectAtIndex:0];
	}
	
	_closeButtonClicked = NO;
	[_tabBarControl setAutomaticallyAnimates:NO];
}

- (id)windowWillReturnFieldEditor:(NSWindow *)sender toObject:(id)anObject
{
	id value = nil;
	
    if (anObject == [[[_tabView selectedTabViewItem] identifier] inputLine]) {
        value = [[[_tabView selectedTabViewItem] identifier] fieldEditor];
    }
	return value;
}

- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)sender
{
	return [[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] undoManager];
}

- (void)windowDidBecomeKey:(NSNotification *)note
{
	if ([[[_tabView selectedTabViewItem] identifier] isInputLineVisible]) {
		[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] _setFirstResponder];
	} else {
		[[self window] makeFirstResponder:[[[_tabView selectedTabViewItem] identifier] transcriptView]];
	}
	
	//Change the target of the close menu item
	NSMenu *fileMenu = [[[NSApp mainMenu] itemAtIndex:1] submenu];
	[[fileMenu itemAtIndex:6] setTarget:self];
}

- (void)windowDidResignKey:(NSNotification *)note
{
	//Reset the target of the close menu item
	NSMenu *fileMenu = [[[NSApp mainMenu] itemAtIndex:1] submenu];
	[[fileMenu itemAtIndex:6] setTarget:nil];
}

#pragma mark -
#pragma mark PSMTabBarControl Delegate Methods
#pragma mark -

- (void)tabView:(NSTabView *)tabView didCloseTabViewItem:(NSTabViewItem *)tabViewItem
{
	if ([[tabViewItem identifier] isTab]) {
		//Do cleanup on the closed tab
		id identifier = [tabViewItem identifier];
		NSWindow *window = [identifier window];
		
		[[identifier peopleDrawer] close];
		[[identifier peopleDrawer] performSelector:@selector(setParentWindow:) withObject:nil afterDelay:2.0];
		
		if ([tabView numberOfTabViewItems] == 0) {
			[_tabBarControl setDelegate:nil];
			if (!_closeButtonClicked) {
				[[self window] close];
			}
		}
		
		[identifier finishChat:self];
		[[[identifier chat] chatDoc] close];
		[identifier setChat:nil];
	}
}

- (void)tabView:(NSTabView *)tabView willSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
	NSDrawer *currentDrawer = [[[tabView selectedTabViewItem] identifier] peopleDrawer];
	
	//close the current drawer, open the new drawer if it is a group chat
	[currentDrawer close];
	if ([[[tabViewItem identifier] chat] style] == 8) {
		[[[tabViewItem identifier] peopleDrawer] open];
	}
}

- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(NSTabViewItem *)tabViewItem
{
	HaxTabViewItem *item = (HaxTabViewItem *)[tabView selectedTabViewItem];
	HaxChatController *controller = [tabViewItem identifier];
	
	//window should be a SecureWindow, but be extra-careful
	if ([[self window] respondsToSelector:@selector(isLockVisible)]) {
		[(SecureWindow *)[self window] showLock:[[controller window] isLockVisible]];
	}/* else {
		NSLog(@"Chax warning: Tab window isn't an instance of SecureWindow.");
	}*/
	
	[[self window] setTitle:[NSString stringWithFormat:LocalizedString(@"tab_window_title"), [item label]]];
	
	if ([controller isInputLineVisible]) {
		NSTextField *inputLine = [[[_tabView selectedTabViewItem] identifier] inputLine];
		[[self window] makeFirstResponder:inputLine];
		[[inputLine currentEditor] setSelectedRange:NSMakeRange([[inputLine stringValue] length], 0)];
	} else {
		[[self window] makeFirstResponder:[controller transcriptView]];
	}
	
	//Reset the highlight status of the chat if it is marked as having new messages
	if ([item chatStatus] == 1) {
		//If the user is offline set the tab status to the offline color. Otherwise set to no status
		[item setChatStatus:([[[controller chat] otherPerson] status] == 1) ? 3 : 0];
		[_tabBarControl setNeedsDisplay];
		
		//Reset unread messages flag
		[[controller chat] setUnreadMessages:NO];
	}
	
	//See if we need to disable the overflow button animation
	NSArray *tabs = [_tabBarControl representedTabViewItems];
	int i;
	BOOL animate = NO;
	
	for (i = [_tabBarControl numberOfVisibleTabs]; i < [tabs count]; i++) {
		HaxTabViewItem *tab = [tabs objectAtIndex:i];
		
		if ([tab chatStatus] == 1) {
			animate = YES;
			break;
		}
	}
	
	if (!animate) {
		[[_tabBarControl overflowPopUpButton] setAnimatingAlternateImage:NO];
	}
	
	[item setLabel:[item label]];
}

- (NSMenu *)tabView:(NSTabView *)tabView menuForTabViewItem:(NSTabViewItem *)tabViewItem
{
	id <NSMenuItem> menuItem;
	NSMenu *menu = [[NSMenu alloc] init];
	
	if ([[[[tabViewItem identifier] chat] chatDoc] presentityToGetInfoOn]) {
		menuItem = [menu addItemWithTitle:[[[[[NSApp mainMenu] itemAtIndex:4] submenu] itemAtIndex:0] title] action:@selector(showSelectedBuddyInfo:) keyEquivalent:@""];
		[menuItem setTag:9202];
		[menuItem setTarget:[[[tabViewItem identifier] chat] chatDoc]];
	}
	
	/*menuItem = [menu addItemWithTitle:LocalizedString(@"move_to_window") action:@selector(moveTabToWindow:) keyEquivalent:@""];
	[menuItem setTag:9200];
	[menuItem setTarget:self];
	[menuItem setRepresentedObject:tabViewItem];*/
	
	menuItem = [menu addItemWithTitle:LocalizedString(@"close_tab") action:@selector(closeTab:) keyEquivalent:@""];
	[menuItem setTag:9201];
	[menuItem setTarget:self];
	[menuItem setRepresentedObject:tabViewItem];
	
	return [menu autorelease];
}

- (BOOL)tabView:(NSTabView*)aTabView shouldDragTabViewItem:(NSTabViewItem *)tabViewItem fromTabBar:(PSMTabBarControl *)tabBarControl
{
	return YES;
}

- (BOOL)tabView:(NSTabView*)aTabView shouldDropTabViewItem:(NSTabViewItem *)tabViewItem inTabBar:(PSMTabBarControl *)tabBarControl
{
	return [Chax boolForKey:@"TabDisableTearOff"] ? tabBarControl != nil : YES;
}

- (void)tabView:(NSTabView*)aTabView didDropTabViewItem:(NSTabViewItem *)tabViewItem inTabBar:(PSMTabBarControl *)tabBarControl
{
	//reset tab stuff
	//[[[tabViewItem identifier] inputLine] setTarget:[tabBarControl delegate]];
	ChatController *controller = [tabViewItem identifier];
	NSWindow *newWindow = [[tabBarControl delegate] window];
	
	[[controller inputLine] setTarget:nil];
	
	NSDrawer *drawer = [[tabViewItem identifier] peopleDrawer];
	
	if ([drawer parentWindow] != newWindow) {
		int state = [drawer state];
		[drawer close];
		[drawer setParentWindow:newWindow];
		
		if (state == NSDrawerOpenState || state == NSDrawerClosingState) {
			[drawer performSelector:@selector(open:) withObject:nil afterDelay:0.5];
		}
		
		//move the ChatMiniwindowController window to the new window
		ChatMiniwindowController *miniwindowController;
		object_getInstanceVariable([[controller chat] chatDoc], "_miniwindowController", (void **)&miniwindowController);
		[miniwindowController autorelease];
		miniwindowController = [[ChatMiniwindowController alloc] initWithWindow:[controller window]];
		[miniwindowController setChat:[controller chat]];
		object_setInstanceVariable([[controller chat] chatDoc], "_miniwindowController", miniwindowController);
	}
}

- (void)tabViewDidChangeNumberOfTabViewItems:(NSTabView *)tabView
{
	if ([tabView numberOfTabViewItems] > 0) {
		NSEnumerator *enumerator = [[tabView tabViewItems] objectEnumerator];
		NSTabViewItem *nextTab;
		
		while ( (nextTab = [enumerator nextObject]) ) {
			[[nextTab identifier] setIsTab:YES];
		}
		
		[_tabBarControl setAutomaticallyAnimates:YES];
	}
}

- (NSArray *)allowedDraggedTypesForTabView:(NSTabView *)aTabView
{
	return [NSArray arrayWithObjects:NSFilenamesPboardType, NSRTFPboardType, NSStringPboardType, NSURLPboardType, nil];
}

- (void)tabView:(NSTabView *)aTabView acceptedDraggingInfo:(id <NSDraggingInfo>)draggingInfo onTabViewItem:(NSTabViewItem *)tabViewItem
{
	[[[[_tabView selectedTabViewItem] identifier] fieldEditor] performDragOperation:draggingInfo];
}

- (NSImage *)tabView:(NSTabView *)aTabView imageForTabViewItem:(NSTabViewItem *)tabViewItem offset:(NSSize *)offset styleMask:(unsigned int *)styleMask
{
	// grabs whole window image
	NSImage *viewImage = [[[NSImage alloc] init] autorelease];
	NSRect contentFrame = [[[self window] contentView] frame];
	[[[self window] contentView] lockFocus];
	NSBitmapImageRep *viewRep = [[[NSBitmapImageRep alloc] initWithFocusedViewRect:contentFrame] autorelease];
	[viewImage addRepresentation:viewRep];
	[[[self window] contentView] unlockFocus];
	
    // grabs snapshot of dragged tabViewItem's view (represents content being dragged)
	NSView *viewForImage = [tabViewItem view];
	NSRect viewRect = [viewForImage frame];
	NSImage *tabViewImage = [[[NSImage alloc] initWithSize:viewRect.size] autorelease];
	[tabViewImage lockFocus];
	[viewForImage drawRect:[viewForImage bounds]];
	[tabViewImage unlockFocus];
	
	[viewImage lockFocus];
	NSPoint tabOrigin = [_tabView frame].origin;
	tabOrigin.x += 10;
	tabOrigin.y += 13;
	[tabViewImage compositeToPoint:tabOrigin operation:NSCompositeSourceOver];
	[viewImage unlockFocus];
	
	id <PSMTabStyle> style = (id <PSMTabStyle>)[[aTabView delegate] style];
	
	//draw over where the tab bar would usually be
	NSRect tabFrame = [_tabBarControl frame];
	[viewImage lockFocus];
	[[NSColor windowBackgroundColor] set];
	NSRectFill(tabFrame);
	//draw the background flipped, which is actually the right way up
	NSAffineTransform *transform = [NSAffineTransform transform];
	[transform scaleXBy:1.0 yBy:-1.0];
	[transform concat];
	tabFrame.origin.y = -tabFrame.origin.y - tabFrame.size.height;
	[style drawBackgroundInRect:tabFrame];
	[transform invert];
	[transform concat];
	
	[viewImage unlockFocus];
	
	if (_tabPosition == BottomPosition) {
		offset->height = contentFrame.size.height;
		offset->width = [(id <PSMTabStyle>)[[aTabView delegate] style] leftMarginForTabBarControl];
	} else if (_tabPosition == TopPosition) {
		offset->height = 21;
		offset->width = [(id <PSMTabStyle>)[[aTabView delegate] style] leftMarginForTabBarControl];
	} else if (_tabPosition == LeftPosition) {
		offset->width = 0;
		offset->height = 22 + [style topMarginForTabBarControl];
	} else {
		offset->width = [_tabBarControl frame].origin.x;
		offset->height = 22 + [style topMarginForTabBarControl];
	}
	
	//offset->height = contentFrame.size.height;
	*styleMask = NSTitledWindowMask | NSTexturedBackgroundWindowMask;
	
	return viewImage;
}

- (PSMTabBarControl *)tabView:(NSTabView *)aTabView newTabBarForDraggedTabViewItem:(NSTabViewItem *)tabViewItem atPoint:(NSPoint)point
{
	//create a new window controller with no tab items
	TabbedChatController *controller = [[TabbedChatController alloc] initWithWindowNibName:@"TabWindow"];
	id <PSMTabStyle> style = (id <PSMTabStyle>)[[aTabView delegate] style];
	
	NSRect windowFrame = [[self window] frame];
	[[controller window] setFrame:windowFrame display:NO];
	
	if (_tabPosition == BottomPosition) {
		point.y += [[[controller window] contentView] frame].size.height;
		point.x -= [style leftMarginForTabBarControl];
	} else if (_tabPosition == TopPosition) {
		point.y += 21;
		point.x -= [style leftMarginForTabBarControl];
	} else if (_tabPosition == LeftPosition) {
		point.y += 21 + [style topMarginForTabBarControl];
	} else {
		point.x -= [_tabBarControl frame].origin.x;
		point.y += 21 + [style topMarginForTabBarControl];
	}
	
	[[controller window] setFrameTopLeftPoint:point];
	[[controller tabBar] setStyle:style];
	
	[[tabViewItem identifier] setIsTab:NO];
	[_subcontrollers addObject:controller];
	
	return [controller tabBar];
}

- (void)tabView:(NSTabView *)aTabView closeWindowForLastTabViewItem:(NSTabViewItem *)tabViewItem
{
	[[self window] close];
}

- (BOOL)tabView:(NSTabView *)aTabView validateOverflowMenuItem:(id <NSMenuItem>)menuItem forTabViewItem:(NSTabViewItem *)tabViewItem
{
	if ([[menuItem representedObject] chatStatus] > 0) {
		NSColor *color = nil;
		
		switch ([[menuItem representedObject] chatStatus]) {
			case 1:
				if ( !(color = [NSUnarchiver unarchiveObjectWithData:[Chax dataForKey:@"TabUnreadColor"]]) ) {
					color = [NSColor redColor];
				}
				break;
			case 2:
				if ( !(color = [NSUnarchiver unarchiveObjectWithData:[Chax dataForKey:@"TabTypingColor"]]) ) {
					color = [NSColor colorWithCalibratedRed:0.0 green:0.5 blue:0.0 alpha:1.0];
				}
				break;
			case 3:
				if ( !(color = [NSUnarchiver unarchiveObjectWithData:[Chax dataForKey:@"TabOfflineColor"]]) ) {
					color = [NSColor darkGrayColor];
				}
				break;
		}
		
		NSAttributedString *attributedTitle = [[NSAttributedString alloc] initWithString:[menuItem title] attributes:[NSDictionary dictionaryWithObjectsAndKeys:color, NSForegroundColorAttributeName, [NSFont fontWithName:@"LucidaGrande" size:14], NSFontAttributeName, nil, nil]];
		[menuItem setAttributedTitle:attributedTitle];
		[attributedTitle release];
	} else {
		[menuItem setAttributedTitle:nil];
	}
	
	[menuItem setImage:nil];
	
	if ([Chax boolForKey:@"TabShowUserIcons"]) {
		NSImage *image = [[[[[tabViewItem identifier] chat] otherPerson] picture] image];
		
		if (image) {
			[menuItem setImage:[image imageScaledSmoothlyToSize:NSMakeSize(15, 15)]];
		}
	}
	
	return YES;
}

#pragma mark -
#pragma mark Forwarded chat controller methods
#pragma mark -

- (void)toggleParticipants:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] toggleParticipants:fp8];
}

- (void)toggleHidePictures:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] toggleHidePictures:fp8];
}

- (void)toggleHideAudioStatus:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] toggleHideAudioStatus:fp8];
}

- (void)toggleHideVideoStatus:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] toggleHideVideoStatus:fp8];
}

- (void)sortByStatus:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] sortByStatus:fp8];
}

- (void)sortByFirstName:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] sortByFirstName:fp8];
}

- (void)sortByLastName:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] sortByLastName:fp8];
}

- (void)addLink:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] addLink:fp8];
}

- (void)composeEMail:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] composeEMail:fp8];
}

- (void)sendSong:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] sendSong:fp8];
}

- (void)sendFile:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] sendFile:fp8];
}

- (void)attachFile:(id)fp8
{
	[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] attachFile:fp8];
}

- (void)insertSmiley:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] insertSmiley:fp8];
}

- (void)setRendererFromMenuItem:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] setRendererFromMenuItem:fp8];
}

- (void)inviteToVideoChat:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] inviteToVideoChat:fp8];
}

- (void)inviteToAudioChat:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] inviteToAudioChat:fp8];
}

- (void)toggleHideSmileys:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] toggleHideSmileys:fp8];
}

- (void)chooseBackgroundImage:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] chooseBackgroundImage:fp8];
}

- (void)clearBackground:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] clearBackground:fp8];
}

- (void)toggleShowNames:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] toggleShowNames:fp8];
}

- (void)toggleShowPictures:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] toggleShowPictures:fp8];
}

- (void)setChatShowsNames:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] setChatShowsNames:fp8];
}

- (void)setChatShowsPictures:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] setChatShowsPictures:fp8];
}

- (void)setChatShowsNamesAndPictures:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] setChatShowsNamesAndPictures:fp8];
}

- (void)chooseChatOptions:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] chooseChatOptions:fp8];
}

- (void)sendMessage:(id)fp8
{
	[(ChatController *)[[_tabView selectedTabViewItem] identifier] sendMessage:fp8];
	[[self window] makeFirstResponder:fp8];
}

- (void)sendPartialMessage:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] sendPartialMessage:fp8];
}

- (void)cancelMessage:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] cancelMessage:fp8];
}

- (void)finishChat:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] finishChat:fp8];
}

- (void)ignore:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] ignore:fp8];
}

- (void)block:(id)fp8
{
	[[[_tabView selectedTabViewItem] identifier] block:fp8];
}

- (void)showSelectedBuddyInfo:(id)fp8
{
	[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] showSelectedBuddyInfo:fp8];
}

- (void)addABuddy:(id)sender
{
	[[Fezz sharedInstance] doAddBuddyCommandForPresentity:[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] presentityToGetInfoOn] withWindow:[self window]];
}

- (void)runPageLayout:(id)sender
{
	[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] runPageLayout:sender];
}

- (void)printDocument:(id)sender
{
	[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] printDocument:sender];
}

- (void)saveDocumentTo:(id)sender
{
	[[[[[_tabView selectedTabViewItem] identifier] chat] chatDoc] saveDocumentTo:sender];
}

@end
