/*
 * URLTextView.m
 *
 * Copyright (c) 2006 Kent Sutherland
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy of
 * this software and associated documentation files (the "Software"), to deal in
 * the Software without restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
 * Software, and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

//Code adapted from http://www.cocoadev.com/index.pl?ClickableUrlInTextView

#import "URLTextView.h"

@implementation URLTextView

- (void)awakeFromNib
{
	[self setDrawsBackground:NO];
	[[self enclosingScrollView] setDrawsBackground:NO];
	
	NSTextStorage *storage = [self textStorage];
	[storage beginEditing];
	[storage addAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[NSURL URLWithString:[storage string]], NSLinkAttributeName,
																		[NSNumber numberWithInt:NSSingleUnderlineStyle], NSUnderlineStyleAttributeName,
																		[NSColor blueColor], NSForegroundColorAttributeName, NULL, NULL]
										range:NSMakeRange(0, [storage length])];
	[storage endEditing];
}

- (void)clickedOnLink:(id)link atIndex:(unsigned)charIndex
{
	[[NSWorkspace sharedWorkspace] openURL:link];
}

/*- (void)drawRect:(NSRect)rect
{
	[super drawRect:rect];
	NSRectFill(rect);
}*/

- (void)highlightLinks
{
	NSTextStorage *textStorage = [self textStorage];
	NSString *string = [textStorage string];
	NSRange searchRange = NSMakeRange(0, [string length]);
	NSRange foundRange;
	BOOL noPrefix = NO;

	[textStorage beginEditing];
	
	//remove previous links
	[textStorage removeAttribute:NSUnderlineStyleAttributeName range:searchRange];
	[textStorage removeAttribute:NSLinkAttributeName range:searchRange];
	[textStorage addAttribute:NSForegroundColorAttributeName value:[self textColor] range:searchRange];
	
	do {
		//We assume that all URLs start with http://
		foundRange = [string rangeOfString:@"http://" options:0 range:searchRange];
		
		//If nothing is found with http://, check www.
		if (foundRange.location == NSNotFound) {
			foundRange = [string rangeOfString:@"www." options:0 range:searchRange];
			noPrefix = YES;
		}

		if (foundRange.length > 0) { //Did we find a URL?
			NSURL *theURL;
			NSRange endOfURLRange;

			//Restrict the searchRange so that it won't find the same string again
			searchRange.location = foundRange.location + foundRange.length;
			searchRange.length = [string length] - searchRange.location;

			//We assume the URL ends with whitespace
			endOfURLRange = [string rangeOfCharacterFromSet:
				[NSCharacterSet whitespaceAndNewlineCharacterSet]
				options:0 range:searchRange];

			//The URL could also end at the end of the text.  The next line fixes it in case it does
			if (endOfURLRange.length == 0) {
				endOfURLRange.location = [string length] - 1;
			}
			
			//Remove any trailing periods from the range
			while ([string characterAtIndex:NSMaxRange(endOfURLRange)] == '.' || [string characterAtIndex:NSMaxRange(endOfURLRange)] == '!' || [string characterAtIndex:NSMaxRange(endOfURLRange)] == '?') {
				endOfURLRange.location--;
			}

			//Set foundRange's length to the length of the URL
			foundRange.length = endOfURLRange.location - foundRange.location + 1;

			//grab the URL from the text
			if (noPrefix) {
				theURL = [NSURL URLWithString:[@"http://" stringByAppendingString:[string substringWithRange:foundRange]]];
			} else {
				theURL = [NSURL URLWithString:[string substringWithRange:foundRange]];
			}

			//Make the link attributes and apply those attributes to the URL in the text
			[textStorage addAttributes:[NSDictionary dictionaryWithObject:theURL forKey:NSLinkAttributeName] range:foundRange];
		}
	
	} while (foundRange.length != 0); //repeat the do block until it no longer finds anything

	[textStorage endEditing];
}

@end
